r"""
Installation requirements:
    pip install trame trame-vuetify pandas
"""

import pandas as pd

from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import html
from trame.widgets import vuetify3 as vuetify

server = get_server(client_type="vue3")
state, ctrl = server.state, server.controller

state.setdefault("active_ui", None)

# --------------------------------------------------------------------------------
# Loading dataframe
# --------------------------------------------------------------------------------
data = [
    {
        "name": "Frozen Yogurt",
        "calories": 200,
        "fat": 6,
        "carbs": 24,
        "protein": 4,
        "iron": "1%",
        "glutenfree": True,
    },
    {
        "name": "Ice cream sandwich",
        "calories": 200,
        "fat": 9,
        "carbs": 37,
        "protein": 4.3,
        "iron": "1%",
        "glutenfree": False,
    },
    {
        "name": "Eclair",
        "calories": 300,
        "fat": 16,
        "carbs": 23,
        "protein": 6,
        "iron": "7%",
        "glutenfree": False,
    },
    {
        "name": "Cupcake",
        "calories": 300,
        "fat": 3.7,
        "carbs": 67,
        "protein": 4.3,
        "iron": "8%",
        "glutenfree": True,
    },
    {
        "name": "Gingerbread",
        "calories": 400,
        "fat": 16,
        "carbs": 49,
        "protein": 3.9,
        "iron": "16%",
        "glutenfree": True,
    },
    {
        "name": "Jelly bean",
        "calories": 400,
        "fat": 0,
        "carbs": 94,
        "protein": 0,
        "iron": "0%",
        "glutenfree": False,
    },
    {
        "name": "Lollipop",
        "calories": 400,
        "fat": 0.2,
        "carbs": 98,
        "protein": 0,
        "iron": "2%",
        "glutenfree": True,
    },
    {
        "name": "Honeycomb",
        "calories": 400,
        "fat": 3.2,
        "carbs": 87,
        "protein": 6.5,
        "iron": "45%",
        "glutenfree": True,
    },
    {
        "name": "Donut",
        "calories": 500,
        "fat": 25,
        "carbs": 51,
        "protein": 4.9,
        "iron": "22%",
        "glutenfree": True,
    },
    {
        "name": "KitKat",
        "calories": 500,
        "fat": 26,
        "carbs": 65,
        "protein": 7,
        "iron": "6%",
        "glutenfree": True,
    },
]


frame = pd.DataFrame.from_dict(data)

# --------------------------------------------------------------------------------
# Configure table columns and options
# --------------------------------------------------------------------------------

# fmt: off
headers = [
    {"key": "name", "title": "Dessert", "align": "start", "sortable": False},
    {"key": "calories", "title": "Calories"},
    {"key": "fat", "title": "Fat (g)"},
    {"key": "carbs", "title": "Carbs (g)"},
    {"key": "protein", "title": "Protein (g)"},
    {"key": "iron", "title": "Iron (%)"},
    {"key": "glutenfree", "title": "Gluten-Free"},
]

state.setdefault("group_by", [{"key": "glutenfree", "order": "asc"}])

# fmt: on
_, rows = vuetify.dataframe_to_grid(frame)


table = {
    "group_by": ("group_by",),
    "headers": ("headers", headers),
    "items": ("rows", rows),
    "search": ("query", ""),
    "classes": "elevation-1 ma-4",
    "multi_sort": True,
    "dense": True,
    "items_per_page": 10,
}


# --------------------------------------------------------------------------------
# GUI
# --------------------------------------------------------------------------------


@state.change("rows")
def update_values(rows, **kwargs):
    print("rows changed")
    # print(rows)


with SinglePageLayout(server) as layout:
    layout.title.set_text("Vuetify table example")
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VTextField(
            v_model=("query", ""),
            placeholder="Search",
            dense=True,
            hide_details=True,
        )

    with layout.content:
        state.rows = rows
        with vuetify.VDataTable(**table):
            with vuetify.Template(
                group_header="{ item, columns, toggleGroup, isGroupOpen }",
                __properties=[
                    ("group_header", "v-slot:group-header"),
                ],
            ):
                with html.Tr():
                    with html.Td(colspan=("columns.length",)):
                        vuetify.VBtn(
                            icon=("isGroupOpen(item) ? '$expand' : '$next'",),
                            size="small",
                            variant="text",
                            click="toggleGroup(item)",
                        )
                        html.Span("{{ item.value ?'Gluten free' : 'Contains gluten' }}")

            with vuetify.Template(
                calories="{ item }",
                __properties=[
                    ("calories", "v-slot:item.calories"),
                ],
            ):
                vuetify.VChip(
                    "{{ item.calories }}",
                    color=("item.color",),
                    dark=True,
                )
            with vuetify.Template(
                glutenfree="{ item }",
                __properties=[
                    ("glutenfree", "v-slot:item.glutenfree"),
                ],
            ):
                vuetify.VCheckbox(
                    "{{ item.glutenfree }}",
                    v_model=("item.glutenfree",),
                    disabled=False,
                    input="flushState('rows')",
                )


if __name__ == "__main__":
    server.start()



import time

from aiohttp import web

from trame.app import get_server
from trame.decorators import TrameApp, controller
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import html
from trame.widgets import vuetify3 as v3


@TrameApp()
class App:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.server.cli.add_argument("--session")
        self._build_ui()

        # Use CLI to know when used inside docker
        args, _ = self.server.cli.parse_known_args()
        session = args.session

        # simple default state
        self.state.response = {}
        self.state.session = session
        self.state.url = "/trame-endpoint"

        # Need to adjust URL when within docker
        if session:
            self.state.url = f"/api/{session}/trame-endpoint"

    @property
    def state(self):
        return self.server.state

    @controller.add("on_server_bind")
    def _bind_routes(self, wslink_server):
        wslink_server.app.add_routes(
            [web.get("/trame-endpoint", self.on_trame_endpoint)]
        )

    def on_trame_endpoint(self, request):
        return web.json_response({"url": self.state.url, "time": time.time()})

    def _build_ui(self):
        with SinglePageLayout(self.server) as layout:
            with layout.toolbar:
                v3.VSpacer()
                v3.VBtn(
                    "Make request",
                    click="utils.get('fetch')(url).then(v => v.json()).then(v => (response = v))",
                )
            with layout.content:
                html.Div("URL: {{ url }}")
                html.Div("Session: {{ session }}")
                html.Div("Response")
                html.Pre("{{ JSON.stringify(response, null, 2) }}")


def main():
    app = App()
    app.server.start()


if __name__ == "__main__":
    main()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-components trame-plotly
"""

import numpy as np
import pandas as pd
import plotly.figure_factory as ff
import plotly.graph_objects as go

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import plotly, trame, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Charts handling
# -----------------------------------------------------------------------------

contour_raw_data = pd.read_json(
    "https://raw.githubusercontent.com/plotly/datasets/master/contour_data.json"
)
scatter_raw_data = pd.read_json(
    "https://raw.githubusercontent.com/plotly/datasets/master/scatter_data.json"
)
polar_data = pd.read_csv(
    "https://raw.githubusercontent.com/plotly/datasets/master/polar_dataset.csv"
)

scatter_data = scatter_raw_data["Data"]


def clean_data(data_in):
    """
    Cleans data in a format which can be conveniently
    used for drawing traces. Takes a dictionary as the
    input, and returns a list in the following format:

    input = {'key': ['a b c']}
    output = [key, [a, b, c]]
    """
    key = list(data_in.keys())[0]
    data_out = [key]
    for i in data_in[key]:
        data_out.append(list(map(float, i.split(" "))))

    return data_out


def create_ternary_fig(width=300, height=300, **kwargs):
    contour_dict = contour_raw_data["Data"]

    # Defining a colormap:
    colors = [
        "#8dd3c7",
        "#ffffb3",
        "#bebada",
        "#fb8072",
        "#80b1d3",
        "#fdb462",
        "#b3de69",
        "#fccde5",
        "#d9d9d9",
        "#bc80bd",
    ]
    colors_iterator = iter(colors)

    fig = go.Figure()

    for raw_data in contour_dict:
        data = clean_data(raw_data)

        a = [inner_data[0] for inner_data in data[1:]]
        a.append(data[1][0])  # Closing the loop

        b = [inner_data[1] for inner_data in data[1:]]
        b.append(data[1][1])  # Closing the loop

        c = [inner_data[2] for inner_data in data[1:]]
        c.append(data[1][2])  # Closing the loop

        fig.add_trace(
            go.Scatterternary(
                text=data[0],
                a=a,
                b=b,
                c=c,
                mode="lines",
                line=dict(color="#444", shape="spline"),
                fill="toself",
                fillcolor=colors_iterator.__next__(),
            )
        )

    fig.update_layout(
        margin=dict(l=50, r=50, t=50, b=50),
        width=width,
        height=height,
    )
    return fig


def create_polar_fig(width=300, height=300, **kwargs):
    fig = go.Figure()
    fig.add_trace(
        go.Scatterpolar(
            r=polar_data["x1"].tolist(),
            theta=polar_data["y"].tolist(),
            mode="lines",
            name="Figure 8",
            line_color="peru",
        )
    )
    fig.add_trace(
        go.Scatterpolar(
            r=polar_data["x2"].tolist(),
            theta=polar_data["y"].tolist(),
            mode="lines",
            name="Cardioid",
            line_color="darkviolet",
        )
    )
    fig.add_trace(
        go.Scatterpolar(
            r=polar_data["x3"].tolist(),
            theta=polar_data["y"].tolist(),
            mode="lines",
            name="Hypercardioid",
            line_color="deepskyblue",
        )
    )

    fig.update_layout(
        # title = 'Mic Patterns',
        margin=dict(l=20, r=20, t=20, b=20),
        showlegend=False,
        width=width,
        height=height,
    )
    return fig


def create_streamline_fig(width=100, height=100, **kwargs):
    x = np.linspace(-3, 3, 100)
    y = np.linspace(-3, 3, 100)
    Y, X = np.meshgrid(x, y)
    u = -1 - X**2 + Y
    v = 1 + X - Y**2

    # Create streamline figure
    fig = ff.create_streamline(
        x.tolist(),
        y.tolist(),
        u.tolist(),
        v.tolist(),
        arrow_scale=0.1,
    )
    fig.update_layout(
        # title = 'Mic Patterns',
        margin=dict(l=20, r=20, t=10, b=10),
        showlegend=False,
        width=width,
        height=height,
    )
    return fig


def create_contour_fig(width=100, height=100, **kwargs):
    fig = go.Figure(
        data=go.Contour(
            z=[
                [2, 4, 7, 12, 13, 14, 15, 16],
                [3, 1, 6, 11, 12, 13, 16, 17],
                [4, 2, 7, 7, 11, 14, 17, 18],
                [5, 3, 8, 8, 13, 15, 18, 19],
                [7, 4, 10, 9, 16, 18, 20, 19],
                [9, 10, 5, 27, 23, 21, 21, 21],
                [11, 14, 17, 26, 25, 24, 23, 22],
            ],
            contours={
                "coloring": "heatmap",
                "showlabels": True,
                "labelfont": {
                    "size": 12,
                    "color": "black",
                },
            },
            line_smoothing=1,
        )
    )
    fig.update_layout(
        # title = 'Mic Patterns',
        margin=dict(l=20, r=20, t=10, b=10),
        showlegend=False,
        width=width,
        height=height,
    )
    return fig


# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("contour_size")
def update_contour_size(contour_size, **kwargs):
    if contour_size is None:
        return

    ctrl.update_contour(create_contour_fig(**contour_size.get("size")))


@state.change("stream_size")
def update_stream_size(stream_size, **kwargs):
    if stream_size is None:
        return

    ctrl.update_stream(create_streamline_fig(**stream_size.get("size")))


@state.change("polar_size")
def update_polar_size(polar_size, **kwargs):
    if polar_size is None:
        return

    ctrl.update_polar(create_polar_fig(**polar_size.get("size")))


@state.change("ternary_size")
def update_ternary_size(ternary_size, **kwargs):
    if ternary_size is None:
        return

    ctrl.update_ternary(create_ternary_fig(**ternary_size.get("size")))


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------


state.trame__title = "Charts"

with SinglePageLayout(server) as layout:
    layout.title.set_text("Many charts")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height"):
            with vuetify.VRow(style="height: 50%;"):
                with vuetify.VCol():
                    with trame.SizeObserver("polar_size"):
                        ctrl.update_polar = plotly.Figure(
                            display_mode_bar=("false",),
                        ).update
                with vuetify.VCol():
                    with trame.SizeObserver("ternary_size"):
                        ctrl.update_ternary = plotly.Figure(
                            display_mode_bar=("false",),
                        ).update
            with vuetify.VRow(style="height: 50%;"):
                with vuetify.VCol():
                    with trame.SizeObserver("contour_size"):
                        ctrl.update_contour = plotly.Figure(
                            display_mode_bar=("false",),
                        ).update
                with vuetify.VCol():
                    with trame.SizeObserver("stream_size"):
                        ctrl.update_stream = plotly.Figure(
                            display_mode_bar=("false",),
                        ).update

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame
"""

from trame.app import get_server

if __name__ == "__main__":
    server = get_server()
    parser = server.cli
    parser.add_argument("-o", "--output", help="Working directory")
    (args, _unknown) = parser.parse_known_args()
    print(args.output)



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/MultiViews/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/ba381b3ad286b6269e181d76d1e13ef0cc2d5bc7

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK Remote rendering"

# -----------------------------------------------------------------------------
# VTK code
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
mapper.SetInputConnection(cone_source.GetOutputPort())

# -----------------------------------------------------------------------------
# Views
# -----------------------------------------------------------------------------

NB_COLS = 4

render_windows = []
html_views = []

PALETTE = [
    (0.5, 0, 0),
    (0, 0.5, 0),
    (0, 0, 0.5),
    (0.5, 0, 0.5),
    (0.5, 0.5, 0),
    (0, 0.5, 0.5),
]
colors = PALETTE + PALETTE

# Build render windows
for color in colors:
    actor = vtkActor()
    actor.SetMapper(mapper)

    renderer = vtkRenderer()
    renderer.SetBackground(*color)
    render_window = vtkRenderWindow()
    render_window.AddRenderer(renderer)

    render_window_interactor = vtkRenderWindowInteractor()
    render_window_interactor.SetRenderWindow(render_window)
    render_window_interactor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

    renderer.AddActor(actor)
    renderer.ResetCamera()
    render_window.Render()

    render_windows.append(render_window)

# -----------------------------------------------------------------------------


@state.change("resolution")
def update_cone(resolution=DEFAULT_RESOLUTION, **kwargs):
    cone_source.SetResolution(resolution)
    ctrl.update_views()


def update_reset_resolution():
    with state:
        state.resolution = DEFAULT_RESOLUTION

    ctrl.update_views()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
            end=ctrl.update_views,
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            container = None
            for idx, render_window in enumerate(render_windows):
                if idx % NB_COLS == 0:
                    container = vuetify.VRow(
                        classes="pa-0 ma-0",
                        style=f"height: {NB_COLS * 100 / len(render_windows)}%;",
                    )
                with container:
                    with vuetify.VCol(classes="pa-0 ma-0"):
                        view = vtk_widgets.VtkRemoteView(
                            render_window, ref=f"view{idx}"
                        )
                        html_views.append(view)
                        ctrl.update_views.add(view.update)
                        ctrl.reset_camera.add(view.reset_camera)

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

# import asyncio
from pathlib import Path

from paraview import simple

from trame.app import get_server
from trame.assets.local import LocalFileManager
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import paraview, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

ICON_SIZE = 16

ASSETS = LocalFileManager(__file__)
ASSETS.url("select_surface_point", "icons/pqSurfaceSelectionPoint.svg")
ASSETS.url("select_surface_cell", "icons/pqSurfaceSelectionCell.svg")
ASSETS.url("select_frustrum_points", "icons/pqFrustumSelectionPoint.svg")
ASSETS.url("select_frustrum_cells", "icons/pqFrustumSelectionCell.svg")
# ASSETS.url("select_poly_points", "icons/pqPolygonSelectSurfacePoint.svg")
# ASSETS.url("select_poly_cells", "icons/pqPolygonSelectSurfaceCell.svg")
ASSETS.url("select_block", "icons/pqSelectBlock.svg")
# ASSETS.url("select_hover_point", "icons/pqSurfaceHoveringPoint.svg")
# ASSETS.url("select_hover_cell", "icons/pqSurfaceHoveringCell.svg")

MODE_TO_METHOD = dict(
    select_surface_point=simple.SelectSurfacePoints,
    select_surface_cell=simple.SelectSurfaceCells,
    select_frustrum_points=simple.SelectPointsThrough,
    select_frustrum_cells=simple.SelectCellsThrough,
    select_block=simple.SelectSurfaceBlocks,
)

# -----------------------------------------------------------------------------
# ParaView code
# -----------------------------------------------------------------------------

DATA_FILE_PATH = str(Path(__file__).parent.parent.parent / "data/can.ex2")

dataset = simple.OpenDataFile(DATA_FILE_PATH)
representation = simple.Show(dataset)
view = simple.Render()


def clear_selection():
    simple.ClearSelection()
    ctrl.view_update()


@ctrl.set("on_selection_change")
def on_box_selection(event):
    fn = MODE_TO_METHOD[state.selection_mode]
    min_x, max_x, min_y, max_y = event.get("selection")
    fn(Rectangle=[int(min_x), int(min_y), int(max_x), int(max_y)], Modifier=None)
    ctrl.view_update()
    state.selection_mode = None


@state.change("selection_mode")
def on_mode_change(selection_mode, **kwargs):
    # Use box for selection
    state.box_selection = selection_mode in [
        "select_surface_point",
        "select_surface_cell",
        "select_frustrum_points",
        "select_frustrum_cells",
        "select_block",
    ]

    # Toggle from interactive to selection
    if selection_mode is None:
        view.InteractionMode = "3D"
    else:
        view.InteractionMode = "Selection"

    # Handle hover with live update
    state.send_mouse = selection_mode in [
        "select_block",
        "select_hover_point",
        "select_hover_cell",
    ]
    # if state.send_mouse:
    #     asynchronous.create_task(animate())

    # Disable active mode
    if selection_mode == "-":
        state.selection_mode = None


# async def animate():
#     while state.send_mouse:
#         await asyncio.sleep(0.1)
#         ctrl.view_update()

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("ParaView Selection")

    with layout.toolbar:
        vuetify.VSpacer()
        with vuetify.VBtnToggle(v_model=("selection_mode", None)):
            for name, value in ASSETS.assets.items():
                with vuetify.VBtn(value=name, small=True):
                    vuetify.VImg(
                        src=value,
                        contain=True,
                        height=ICON_SIZE,
                        width=ICON_SIZE,
                    )
            with vuetify.VBtn(small=True, click=clear_selection, value="-"):
                vuetify.VIcon("mdi-trash-can-outline", small=True)

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            html_view = paraview.VtkRemoteView(
                view,
                ref="view",
                interactive_ratio=1,
                enable_picking=("send_mouse", False),
                box_selection=("box_selection", False),
                box_selection_change=(ctrl.on_selection_change, "[$event]"),
            )
            ctrl.view_reset_camera = html_view.reset_camera
            ctrl.view_update = html_view.update


def main():
    server.start()


if __name__ == "__main__":
    main()



def test_singleton():
    from trame.app.singleton import Singleton

    @Singleton
    def new_object():
        return object()

    # Normally, a new object would be returned each time
    # However, since this is a singleton, the same object should be returned
    assert new_object() is new_object()



import pyvista as pv
from pyvista import examples

from trame.app import get_server
from trame.decorators import TrameApp, change
from trame.ui.quasar import QLayout
from trame.widgets import html, quasar, vtk

DEFAULT_CAMERA = [
    (567000.9232163235, 5119147.423216323, 6460.423216322832),
    (562835.0, 5114981.5, 2294.5),
    (-0.4082482904638299, -0.40824829046381844, 0.8164965809277649),
]


def color_to_rgb_float(color_str):
    red = int(color_str[1:3], 16)
    green = int(color_str[3:5], 16)
    blue = int(color_str[5:7], 16)
    return (
        float(red) / 255.0,
        float(green) / 255.0,
        float(blue) / 255.0,
    )


def color_to_rgb_hex(color_float):
    return "#{:02x}{:02x}{:02x}".format(
        int(255 * color_float[0]), int(255 * color_float[1]), int(255 * color_float[2])
    )


@TrameApp()
class StHelens:
    def __init__(self, server=None):
        server = get_server(server, client_type="vue3")
        self.server = server
        self._ui = None

        # setup VTK scene
        ds = examples.download_st_helens().warp_by_scalar()
        self.ploter = pv.Plotter()
        self.actor = self.ploter.add_mesh(ds, smooth_shading=True, lighting=True)
        self.ploter.camera_position = DEFAULT_CAMERA
        self.render_window = self.ploter.ren_win
        self.actor.prop.RenderPointsAsSpheresOn()
        self.actor.prop.SetPointSize(4)

        # Set initial values
        self.server.state.update(
            dict(
                actor_edges_show=self.actor.prop.show_edges,
                actor_opacity=self.actor.prop.opacity,
                actor_lighting=self.actor.prop.lighting,
                actor_culling_front=bool(self.actor.prop.GetFrontfaceCulling()),
                actor_culling_back=bool(self.actor.prop.GetBackfaceCulling()),
                actor_ambient=self.actor.prop.ambient,
                actor_diffuse=self.actor.prop.diffuse,
                actor_specular=self.actor.prop.specular,
                actor_specular_power=self.actor.prop.specular_power,
                actor_edge_color=color_to_rgb_hex(self.actor.prop.GetEdgeColor()),
            )
        )

        # Create ui
        self.ui

    @property
    def ctrl(self):
        return self.server.controller

    @property
    def ui(self):
        if self._ui is None:
            with QLayout(self.server, view="hHh lpR fFf") as layout:
                self._ui = layout

                with quasar.QHeader(elevated=True):
                    with quasar.QToolbar():
                        with quasar.QToolbarTitle() as title:
                            quasar.QBtn(
                                dense=True,
                                flat=True,
                                round=True,
                                icon="menu",
                                click="show_drawer = !show_drawer",
                            )
                            title.add_child("VTK - Mt St Helens")
                        quasar.QToggle(
                            v_model=("dark", False),
                            color="black",
                            checked_icon="dark_mode",
                            unchecked_icon="light_mode",
                            update_model_value="utils.quasar.Dark.set($event)",
                        )

                with quasar.QDrawer(
                    v_model=("show_drawer", True),
                    side="left",
                    overlay=False,
                    bordered=True,
                    width=350,
                ):
                    with quasar.QTabs(
                        v_model=("tab_active", "scene"),
                        inline_label=True,
                    ):
                        quasar.QTab(name="scene", label="Scene controller")
                        quasar.QTab(name="actor", label="Actor properties")

                    with quasar.QTabPanels(
                        v_model=("tab_active", "scene"),
                        animated=True,
                        classes="shadow-2 rounded-borders",
                    ):
                        # Scene tab
                        with quasar.QTabPanel(name="scene"):
                            with html.Div(classes="column"):
                                quasar.QToggle(
                                    label="Orientation Widget",
                                    v_model=("scene_orientation_widget", True),
                                )
                                quasar.QBtn(
                                    "Reset Camera",
                                    click=self.ctrl.view_reset_camera,
                                    classes="q-my-sm",
                                )
                                with quasar.QInput(
                                    filled=True,
                                    v_model=("scene_bg_color", "#ffffff"),
                                    dense=True,
                                    classes="q-my-sm",
                                    style=("{ background: scene_bg_color }",),
                                ):
                                    with html.Template(
                                        v_slot_append=True,
                                        __properties=[
                                            ("v_slot_append", "v-slot:append")
                                        ],
                                    ):
                                        with quasar.QIcon(
                                            name="colorize", classes="cursor-pointer"
                                        ):
                                            with quasar.QPopupProxy(
                                                cover=True,
                                                transition_show="scale",
                                                transition_hide="scale",
                                            ):
                                                quasar.QColor(v_model="scene_bg_color")

                        # Actor tab
                        with quasar.QTabPanel(name="actor"):
                            quasar.QItemLabel(
                                "Opacity: {{ actor_opacity }}",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QSlider(
                                v_model=("actor_opacity", 1),
                                min=(0,),
                                max=(1,),
                                step=(0.1,),
                            )
                            quasar.QToggle(
                                label="Lighting", v_model=("actor_lighting", True)
                            )
                            quasar.QSelect(
                                label="Interpolation",
                                classes="q-mb-md",
                                v_model=("actor_interpolation", "Flat"),
                                dense=True,
                                options=(
                                    "actor_interpolation_options",
                                    ["Flat", "Phong"],
                                ),
                            )
                            quasar.QToggle(
                                label="Show Edges",
                                v_model=("actor_edges_show", False),
                            )

                            quasar.QItemLabel(
                                "Edges color",
                                v_show="actor_edges_show",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QColor(
                                v_show="actor_edges_show",
                                v_model="actor_edge_color",
                                default_view="palette",
                                no_footer=True,
                                no_header_tabs=True,
                            )
                            quasar.QSelect(
                                label="Representation",
                                classes="q-mb-md",
                                v_model=("actor_representation", "Surface"),
                                dense=True,
                                options=(
                                    "actor_representation_options",
                                    ["Surface", "Wireframe", "Points"],
                                ),
                            )
                            with html.Div(classes="row"):
                                quasar.QToggle(
                                    label="Frontface Culling",
                                    v_model=("actor_culling_front", False),
                                    classes="col",
                                )
                                quasar.QToggle(
                                    label="Backface Culling",
                                    v_model=("actor_culling_back", False),
                                    classes="col",
                                )

                            quasar.QItemLabel(
                                "Ambient: {{actor_ambient}}",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QSlider(
                                v_model=("actor_ambient", 1),
                                min=(0,),
                                max=(1,),
                                step=(0.1,),
                            )
                            quasar.QItemLabel(
                                "Diffuse: {{actor_diffuse}}",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QSlider(
                                v_model=("actor_diffuse", 1),
                                min=(0,),
                                max=(1,),
                                step=(0.1,),
                            )
                            quasar.QItemLabel(
                                "Specular: {{actor_specular}}",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QSlider(
                                v_model=("actor_specular", 1),
                                min=(0,),
                                max=(1,),
                                step=(0.1,),
                            )
                            quasar.QItemLabel(
                                "Specular Power: {{actor_specular_power}}",
                                caption=True,
                                classes="q-mt-sm",
                            )
                            quasar.QSlider(
                                label="hello",
                                v_model=("actor_specular_power", 1),
                                min=(0,),
                                max=(100,),
                                step=(1,),
                            )

                with quasar.QPageContainer(classes="fullscreen", style="z-index: 0;"):
                    rw = self.render_window
                    with vtk.VtkRemoteView(rw, interactive_ratio=1) as view:
                        # with vtk.VtkLocalView(rw, interactive_ratio=1) as view:
                        self.ctrl.view_update = view.update
                        self.ctrl.view_reset_camera = view.reset_camera

        return self._ui

    @change("scene_bg_color")
    def on_scene_bg_color_change(self, scene_bg_color, **kwargs):
        color = color_to_rgb_float(scene_bg_color)
        for renderer in self.ploter.renderers:
            renderer.SetBackground(color)
        self.ctrl.view_update()

    @change("scene_orientation_widget")
    def on_axis_visiblity_change(self, scene_orientation_widget, **kwargs):
        for renderer in self.ploter.renderers:
            if scene_orientation_widget:
                renderer.show_axes()
            else:
                renderer.hide_axes()
        self.ctrl.view_update()

    @change("actor_edges_show")
    def on_actor_edges_show_change(self, actor_edges_show, **kwargs):
        self.actor.prop.show_edges = actor_edges_show
        self.ctrl.view_update()

    @change("actor_opacity")
    def on_actor_opacity_change(self, actor_opacity, **kwargs):
        self.actor.prop.opacity = actor_opacity
        self.ctrl.view_update()

    @change("actor_lighting")
    def on_actor_lighting_change(self, actor_lighting, **kwargs):
        self.actor.prop.lighting = actor_lighting
        self.ctrl.view_update()

    @change("actor_interpolation")
    def on_actor_interpolation(self, actor_interpolation, **kwargs):
        if actor_interpolation == "Flat":
            self.actor.prop.SetInterpolationToFlat()
        if actor_interpolation == "Phong":
            self.actor.prop.SetInterpolationToPhong()
        self.ctrl.view_update()

    @change("actor_edge_color")
    def on_actor_edge_color_change(self, actor_edge_color, **kwargs):
        color = color_to_rgb_float(actor_edge_color)
        self.actor.prop.edge_color = color
        self.ctrl.view_update()

    @change("actor_representation")
    def on_actor_representation_change(self, actor_representation, **kwargs):
        if actor_representation == "Points":
            self.actor.prop.SetRepresentationToPoints()
        if actor_representation == "Wireframe":
            self.actor.prop.SetRepresentationToWireframe()
        if actor_representation == "Surface":
            self.actor.prop.SetRepresentationToSurface()
        self.ctrl.view_update()

    @change("actor_culling_front")
    def on_actor_culling_front_change(self, actor_culling_front, **kwargs):
        self.actor.prop.SetFrontfaceCulling(actor_culling_front)
        self.ctrl.view_update()

    @change("actor_culling_back")
    def on_actor_culling_back_change(self, actor_culling_back, **kwargs):
        self.actor.prop.SetBackfaceCulling(actor_culling_back)
        self.ctrl.view_update()

    @change("actor_ambient")
    def on_actor_ambient_change(self, actor_ambient, **kwargs):
        self.actor.prop.ambient = actor_ambient
        self.ctrl.view_update()

    @change("actor_diffuse")
    def on_actor_diffuse_change(self, actor_diffuse, **kwargs):
        self.actor.prop.diffuse = actor_diffuse
        self.ctrl.view_update()

    @change("actor_specular")
    def on_actor_specular_change(self, actor_specular, **kwargs):
        self.actor.prop.specular = actor_specular
        self.ctrl.view_update()

    @change("actor_specular_power")
    def on_actor_specular_power_change(self, actor_specular_power, **kwargs):
        self.actor.prop.specular_power = actor_specular_power
        self.ctrl.view_update()


if __name__ == "__main__":
    app = StHelens()
    app.server.start()



#!/usr/bin/env -S uv run --script
# -----------------------------------------------------------------------------
# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "pandas",
#     "plotly",
#     "trame-plotly",
#     "trame-vuetify",
#     "trame[app]",
# ]
# ///
# -----------------------------------------------------------------------------
"""
Trame Plotly Chart Selector

Running with uv
   uv run ./00_plotly_charts_selector.py
   or ./00_plotly_charts_selector.py

Required Packages:
   pip install "trame[app]" trame-vuetify trame-plotly plotly pandas jupyterlab

Run as a Desktop Application:
   python 00_plotly_charts_selector.py --app

Run in Jupyter Lab / Notebook:
   Make sure this script ('00_plotly_charts_selector.py') is in the same directory as your notebook,
   or in a directory included in Python's path.
   Then, in a cell, execute:

   from 00_plotly_charts_selector import PlotlyViewer
   PlotlyViewer()

Run as a Web Application (default):
   python 00_plotly_charts_selector.py --server
   # Access via the URLs provided in the console (e.g., http://localhost:8080)
"""

from trame.app import TrameApp
from trame.decorators import change
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vuetify3, plotly

import plotly.express as px
import plotly.graph_objects as go

# -----------------------------------------------------------------------------
# Sample Plotly figures
# -----------------------------------------------------------------------------
SCATTER_MATRIX = px.scatter_matrix(
    px.data.iris(),
    dimensions=["sepal_width", "sepal_length", "petal_width", "petal_length"],
    color="species",
)
SCATTER_3D = px.scatter_3d(
    px.data.iris(),
    x="sepal_length",
    y="sepal_width",
    z="petal_width",
    color="petal_length",
    symbol="species",
)
BAR_CHART = go.Figure(
    data=[go.Bar(x=[1, 2, 3], y=[1, 3, 2])], layout_title_text="A Bar Chart"
)
CONTOUR_PLOT = go.Figure(
    data=[
        go.Contour(
            z=[
                [10, 10.625, 12.5, 15.625, 20],
                [5.625, 6.25, 8.125, 11.25, 15.625],
                [2.5, 3.125, 5.0, 8.125, 12.5],
                [0.625, 1.25, 3.125, 6.25, 10.625],
                [0, 0.625, 2.5, 5.625, 10],
            ]
        )
    ]
)
CONTOUR_PLOT.update_layout(title_text="Contour Plot")

PLOTS = {
    "Contour": CONTOUR_PLOT,
    "Scatter3D": SCATTER_3D,
    "ScatterMatrix": SCATTER_MATRIX,
    "BarChart": BAR_CHART,
}

# -----------------------------------------------------------------------------
# Trame Application
# -----------------------------------------------------------------------------


class PlotlyViewer(TrameApp):
    def __init__(self, server=None):
        super().__init__(server, client_type="vue3")
        self._build_ui()

    @change("active_plot_name")
    def update_plot_figure(self, active_plot_name, **_):
        figure_data = PLOTS.get(active_plot_name)
        if figure_data:
            self.ctx.plotly_display.update(figure_data)

    def _build_ui(self):
        with SinglePageLayout(self.server, full_height=True) as self.ui:
            self.ui.title.set_text("Trame ❤️ Plotly")
            self.ui.icon.hide()

            with self.ui.toolbar:
                vuetify3.VSpacer()
                vuetify3.VSelect(
                    v_model=("active_plot_name", "Contour"),
                    items=("plots", list(PLOTS.keys())),
                    hide_details=True,
                    density="compact",
                    style="max-width: 200px;",
                    variant="outlined",
                    classes="mr-4",
                )

            with self.ui.content:
                plotly.Figure(
                    ctx_name="plotly_display",
                    display_logo=False,
                    display_mode_bar=True,
                )


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------


def main(**kwargs):
    app = PlotlyViewer()  # Instantiate the class, passing the server
    app.server.start(**kwargs)  # Start the specific server instance


if __name__ == "__main__":
    main()



from trame.app import TrameApp
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import html, vtk, vuetify3


class Cone(TrameApp):
    def __init__(self, name=None):
        super().__init__(server=name)
        self._build_ui()

    @property
    def resolution(self):
        return self.state.resolution

    @resolution.setter
    def resolution(self, v):
        with self.state:
            self.state.resolution = v

    def reset_resolution(self):
        self.resolution = 6

    def _build_ui(self):
        with SinglePageLayout(self.server) as self.ui:
            self.ui.icon.click = self.ctrl.reset_camera
            with self.ui.toolbar:
                html.Div("resolution: {{ resolution }}")
                vuetify3.VSpacer()
                vuetify3.VSlider(
                    v_model=("resolution", 6), min=3, max=60, step=1, hide_details=True
                )
                vuetify3.VBtn("Reset", click=self.reset_resolution)
            with self.ui.content:
                with vuetify3.VContainer(fluid=True, classes="pa-0 fill-height"):
                    with vtk.VtkView() as vtk_view:
                        self.ctrl.reset_camera = vtk_view.reset_camera
                        with vtk.VtkGeometryRepresentation():
                            vtk.VtkAlgorithm(
                                vtkClass="vtkConeSource", state=("{ resolution }",)
                            )



from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html, trame, vtk

server = get_server()

# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

with DivLayout(server) as layout:
    container = layout.root
    trame.LifeCycleMonitor(events=("['created']",))
    container.style = "width: 100vw; height: 100vh;"
    with vtk.VtkView(ref="view"):
        with vtk.VtkGeometryRepresentation():
            vtk.VtkAlgorithm(vtk_class="vtkConeSource", state=("{ resolution }",))
    html.Input(
        type="range",
        min=3,
        max=60,
        step=1,
        v_model=("resolution", 6),
        style="position: absolute; top: 20px; left: 20px; z-index: 1; width: 25%; min-width: 300px;",
    )

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify
"""

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, vuetify

server = get_server(client_type="vue2")
state = server.state

# Initial state values
DEFAULT_VALUE = 5
state.trame_title = "Counter"


# Updates
def increment():
    state.my_number += 1


def decrement():
    state.my_number -= 1


@state.change("my_number")
def validate_my_number(my_number, **kwargs):
    if isinstance(my_number, int):
        # Prevent infinite loop
        return

    try:
        state.my_number = int(my_number)
    except ValueError:
        state.my_number = DEFAULT_VALUE


with SinglePageLayout(server) as layout:
    layout.title.set_text("Simple Counter Demo")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VBtn("-", click=decrement)
        vuetify.VBtn("+", click=increment)

    with layout.content:
        with html.Div(classes="ma-8"):
            vuetify.VTextField(v_model=("my_number", DEFAULT_VALUE))
            vuetify.VSlider(v_model=("my_number",))


if __name__ == "__main__":
    server.start()



import asyncio

import pyvista as pv
from pyvista.trame.ui import plotter_ui

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout

pv.OFF_SCREEN = True

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

cyl = pv.Cylinder(resolution=8, direction=(0, 1, 0))

pl = pv.Plotter()
pl.background_color = (0.1, 0.2, 0.4)
pl.add_mesh(cyl, name="cyl")

keep_rotate = True


@ctrl.add_task("on_server_ready")
async def _update(**kwargs):
    await asyncio.sleep(5)
    while keep_rotate:
        cyl.rotate_x(10.0, inplace=True)
        cyl.rotate_y(-5.0, inplace=True)
        pl.add_mesh(cyl, name="cyl")
        pl.render()
        ctrl.view_update()
        await asyncio.sleep(0.5)


with SinglePageLayout(server) as layout:
    with layout.content:
        view = plotter_ui(pl)
        ctrl.view_update = view.update

server.start()



from trame_common.assets.local import (
    LocalFileManager,
    to_base64,
    to_mime,
    to_txt,
    to_url,
)

__all__ = [
    "LocalFileManager",
    "to_mime",
    "to_txt",
    "to_base64",
    "to_url",
]



from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import vuetify

with DivLayout(get_server(client_type="vue2")) as layout:
    vuetify.VTextField(
        label="defectName",
        v_model=("defectForm",),
        required=True,
        __properties=["required"],
    )
    print(layout)



import os
import sys

import vtk
import zarr
from vtk.util import numpy_support as np_s


class vtkContourGeneratorFromZarr(object):
    def __init__(self, basepath):
        self.zarrPath = os.path.normpath(basepath)
        if not os.path.exists(self.zarrPath):
            print(f"Path ({self.zarrPath}) is not valid")
            sys.exit(1)

        # set up zarr stuff
        self.zarrStore = zarr.open(self.zarrPath, mode="r")
        self.multiscales = self.zarrStore.attrs["multiscales"][0]
        self.datasets = self.multiscales["datasets"]
        self.root = self.zarrStore["/"]
        self.maxLevel = len(self.datasets) - 1

        self.contour = vtk.vtkFlyingEdges3D()
        self.contour.ComputeScalarsOff()
        self.contour.ComputeNormalsOn()
        self.contour.ComputeGradientsOff()

    def getVolume(self, scale, dataset):
        origin = self.getScaleOrigin(scale)
        spacing = self.getScaleSpacing(scale)

        path = dataset["path"]
        dims = self.root[path].attrs["_ARRAY_DIMENSIONS"]
        shape = self.root[path].shape
        xIdx = dims.index("x")
        yIdx = dims.index("y")
        zIdx = dims.index("z")

        ds = vtk.vtkImageData()
        ds.SetOrigin(origin)
        ds.SetSpacing(spacing)
        ds.SetDimensions(shape[xIdx], shape[yIdx], shape[zIdx])

        array = np_s.numpy_to_vtk(self.root[path][:].flatten())
        array.SetName("Content")
        ds.GetPointData().SetScalars(array)

        return ds

    def getMaxLevel(self):
        return self.maxLevel

    def getScaleOrigin(self, scale):
        origin = [0.0, 0.0, 0.0]
        origin[0] = self.root[str(scale) + "/x"][0]
        origin[1] = self.root[str(scale) + "/y"][0]
        origin[2] = self.root[str(scale) + "/z"][0]
        return origin

    def getScaleSpacing(self, scale):
        spacing = [0.0, 0.0, 0.0]
        spacing[0] = self.root[str(scale) + "/x"][1] - self.root[str(scale) + "/x"][0]
        spacing[1] = self.root[str(scale) + "/y"][1] - self.root[str(scale) + "/y"][0]
        spacing[2] = self.root[str(scale) + "/z"][1] - self.root[str(scale) + "/z"][0]
        return spacing

    def contourForLevel(self, level, contours=[1]):
        # Update contours values
        ctrIdx = 0
        self.contour.SetNumberOfContours(len(contours))
        for value in contours:
            self.contour.SetValue(ctrIdx, value)
            ctrIdx += 1

        if level > self.maxLevel:
            print(f"level {level} > max level {self.maxLevel}")

        volume = self.getVolume(level, self.datasets[level])
        self.contour.SetInputData(volume)
        self.contour.Update()

        _ds = self.contour.GetOutput()
        nb_points = _ds.GetNumberOfPoints()
        nb_cells = _ds.GetNumberOfCells()
        return nb_points, nb_cells

    def getContour(self):
        return self.contour



r"""
Multi-server setup

Installation requirements:
    pip install trame trame-vuetify
"""

import asyncio

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

server1 = get_server("server1", client_type="vue2")
state1 = server1.state
state1.trame__title = "Server1"

server2 = get_server("server2", client_type="vue2")
state2 = server2.state
state2.trame__title = "Server2"


def send_to_server1(message):
    with state1:
        state1.received = message


def send_to_server2(message):
    with state2:
        state2.received = message


with SinglePageLayout(server1) as layout:
    layout.title.set_text("Server1")

    with layout.content:
        with vuetify.VCard(classes="pa-4"):
            vuetify.VTextField(
                v_model=("message", "Hi from server 1"),
            )
            vuetify.VBtn(
                "Send message to server 2",
                click=(send_to_server2, "[message]"),
            )
            vuetify.VTextField(readonly=True, v_model=("received", "<message here>"))

with SinglePageLayout(server2) as layout:
    layout.title.set_text("Server2")

    with layout.content:
        with vuetify.VCard(classes="pa-4"):
            vuetify.VTextField(
                v_model=("message", "Hi from server 2"),
            )
            vuetify.VBtn(
                "Send message to server 1",
                click=(send_to_server1, "[message]"),
            )
            vuetify.VTextField(
                readonly=True,
                v_model=("received", "<message here>"),
            )


if __name__ == "__main__":
    task1 = server1.start(exec_mode="task", port=8080)
    task2 = server2.start(exec_mode="task", port=8081)
    loop = asyncio.get_event_loop()
    loop.run_until_complete(asyncio.gather(task1, task2))



from pathlib import Path

from trame.app import get_server
from trame.ui.html import DivLayout

module = {
    "serve": {
        "tailwind": str(Path(__file__).parent.resolve()),
    },
    "scripts": [
        "tailwind/tailwind.js",
    ],
}

HTML_CONTENT = Path(__file__).with_name("content.html").read_text()
CLASSES = "bg-white dark:bg-gray-900 dark:before:fixed dark:before:-z-50 dark:before:inset-0 dark:before:bg-gray-950/50"


class App:
    def __init__(self, server=None, table_size=10):
        self.server = get_server(server, client_type="vue3")
        self.server.enable_module(module)
        self._build_ui()

    def _build_ui(self):
        with DivLayout(self.server) as layout:
            layout.root.classes = CLASSES
            layout.root.add_child(HTML_CONTENT)


def main():
    app = App()
    app.server.start()


if __name__ == "__main__":
    main()



"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-components trame-rca
"""

import asyncio

import paraview.web.venv
from paraview import simple

from trame.app import asynchronous, get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import html, paraview, rca, trame, vuetify

# -----------------------------------------------------------------------------
# Global helpers
# -----------------------------------------------------------------------------

PXM = simple.servermanager.ProxyManager()
WAVELET_SCALAR_RANGE = [37.35310363769531, 276.8288269042969]


def generate_contour_values(data_range, number_of_contours):
    delta = (data_range[1] - data_range[0]) / (number_of_contours - 1)
    return [data_range[0] + (delta * float(i)) for i in range(number_of_contours)]


def id_to_proxy(_id):
    try:
        _id = int(_id)
    except Exception:
        return None
    if _id <= 0:
        return None

    return simple.servermanager._getPyProxy(
        simple.servermanager.ActiveConnection.Session.GetRemoteObject(_id)
    )


# -----------------------------------------------------------------------------
# Wavelet Application
# -----------------------------------------------------------------------------
class App:
    def __init__(self, server=None):
        server = get_server(server, client_type="vue2")

        self.server = server
        self.state = server.state
        self.ctrl = server.controller

        # ParaView Async
        self._running = True
        self._ready = False

        # internal state
        self.hidden_pipeline_proxy_ids = []
        self.active_proxy = None
        self.active_representation = None
        self.active_view = None

        # initial state
        self.state.target_fps = 30  # suggested for jpeg.
        self.state.wavelet_size = 10
        self.state.contours_count = 10
        self.state.working_proxy = ""
        self.state.wavelet_id = 0
        self.state.contour_id = 0
        self.state.clip_id = 0
        self.state.threshold_id = 0
        self.state.active_data_info = None

        # controller
        self.ctrl.on_server_ready.add(self.initialize)

        # state listeners
        self.state.change("contours_count")(self.ui_state_contours_update)
        self.state.change("wavelet_size")(self.ui_state_wavelet_update)
        self.state.change("clip_x_origin")(self.ui_state_clip_update)
        self.state.change("slice_x_origin")(self.ui_state_slice_update)
        self.state.change("threshold_range")(self.ui_state_threshold_update)
        self.state.change("target_fps")(self.ui_state_target_fps_change)

    # ---------------------------------------------------------------
    # Instance life cycle
    # ---------------------------------------------------------------

    def initialize(self, **kwargs):
        # Tasks to monitor state change
        asynchronous.create_task(self.monitor_server_status())

        # RemoteControllerArea
        # self._view_handler = ViewAdapter(self.active_view, "view")
        # self.ctrl.rc_area_register(self._view_handler)

        self._ready = True

        print("Server ready")
        self._push_pipeline_to_ui()
        self.ctrl.view_reset_camera()

    # ---------------------------------------------------------------
    # Background async monitoring tasks
    # ---------------------------------------------------------------

    async def monitor_server_status(self):
        while self._running:
            with self.state as state:
                await asyncio.sleep(1 / self.state.target_fps)

                # Spinning
                if state.spinning and self.active_view:
                    self.active_view.GetActiveCamera().Azimuth(1)
                    self.ctrl.view_update()

                # Update client
                state.status_server += 5
                if state.status_server > 360:
                    state.status_server = 0

    # ---------------------------------------------------------------
    # General API
    # ---------------------------------------------------------------

    def setup_demo(self):
        view = simple.GetRenderView()
        hide_list = []

        # wavelet ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        self.wavelet = simple.Wavelet()
        rep = simple.Show()
        rep.Representation = "Outline"

        # Contour ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        self.contour = simple.Contour(
            Input=self.wavelet,
            Isosurfaces=generate_contour_values(WAVELET_SCALAR_RANGE, 10),
            ContourBy=["POINTS", "RTData"],
        )
        rep = simple.Show()
        simple.Hide(self.contour)
        hide_list.append(rep)

        # Clip ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        self.clip = simple.Clip(
            Input=self.contour,
        )
        rep = simple.Show()

        # Slice ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        self.slice = simple.Slice(
            Input=self.wavelet,
            SliceType="Plane",
        )
        self.slice.SliceType.Normal = [0, 1, 0]
        rep = simple.Show()

        # Threshold ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        self.threshold = simple.Threshold(
            Input=self.wavelet,
            Scalars=["POINTS", "RTData"],
            LowerThreshold=180,
            UpperThreshold=240,
        )
        rep = simple.Show()

        # Plane ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        # create a plane source as a fake widget of the clip
        # make sure to keep them synchronized
        self.plane = simple.Plane(
            Origin=[0, -10, -10],
            Point1=[0, -10, 10],
            Point2=[0, 10, -10],
            XResolution=10,
            YResolution=10,
        )
        rep = simple.Show()
        rep.Representation = "Wireframe"
        self.hidden_pipeline_proxy_ids.append(self.plane.GetGlobalIDAsString())

        # Keep track of view and ids
        self.active_view = view
        self.state.wavelet_id = self.wavelet.GetGlobalIDAsString()
        self.state.contour_id = self.contour.GetGlobalIDAsString()
        self.state.clip_id = self.clip.GetGlobalIDAsString()
        self.state.slice_id = self.slice.GetGlobalIDAsString()
        self.state.threshold_id = self.threshold.GetGlobalIDAsString()

        # Activate wavelet by default
        simple.SetActiveSource(self.wavelet)

    def _push_pipeline_to_ui(self):
        sources = []
        proxies = PXM.GetProxiesInGroup("sources")
        view_proxy = simple.GetActiveView()

        node_map = {}
        for key in proxies:
            proxy = proxies[key]

            source = {"parent": "0"}
            source["name"] = key[0]
            source["id"] = key[1]

            representation = simple.GetRepresentation(proxy=proxy, view=view_proxy)
            source["rep"] = representation.GetGlobalIDAsString()
            source["visible"] = int(representation.Visibility)

            if hasattr(proxy, "Input") and proxy.Input:
                inputProp = proxy.Input
                if hasattr(inputProp, "GetNumberOfProxies"):
                    numProxies = inputProp.GetNumberOfProxies()
                    if numProxies > 1:
                        source["multiparent"] = numProxies
                        for inputIdx in range(numProxies):
                            proxyId = inputProp.GetProxy(inputIdx).GetGlobalIDAsString()
                            if inputIdx == 0:
                                source["parent"] = proxyId
                            else:
                                source[f"parent_{inputIdx}"] = proxyId
                    elif numProxies == 1:
                        source["parent"] = inputProp.GetProxy(0).GetGlobalIDAsString()
                else:
                    source["parent"] = inputProp.GetGlobalIDAsString()

            if key[1] not in self.hidden_pipeline_proxy_ids:
                sources.append(source)

            node_map[source["id"]] = source

        with self.state as state:
            state.git_tree_sources = sources
            state.git_tree_actives = [simple.GetActiveSource().GetGlobalIDAsString()]

    def _push_data_info(self):
        proxy = simple.GetActiveSource()
        info = proxy.GetDataInformation()
        with self.state as state:
            state.active_data_info = dict(
                points=info.GetNumberOfPoints(), cells=info.GetNumberOfCells()
            )

    # ---------------------------------------------------------------
    # GUI callbacks
    # ---------------------------------------------------------------

    def ui_event_pipeline_update(self, active):
        proxy = id_to_proxy(active[0])
        simple.SetActiveSource(proxy)
        self._push_pipeline_to_ui()
        self._push_data_info()

    def ui_event_pipeline_visibility_update(self, selection):
        proxy_id = selection.get("id")
        visible = selection.get("visible")
        proxy = id_to_proxy(proxy_id)
        rep = simple.GetRepresentation(proxy)
        rep.Visibility = visible
        self._push_pipeline_to_ui()
        self.ctrl.view_update()

    def ui_state_contours_update(self, contours_count, **kwargs):
        if not self._ready:
            return

        self.contour.Isosurfaces = generate_contour_values(
            WAVELET_SCALAR_RANGE, contours_count
        )
        self.ctrl.view_update()
        self._push_data_info()

    def ui_state_wavelet_update(self, wavelet_size, **kwargs):
        if not self._ready:
            return

        self.wavelet.WholeExtent = [
            -wavelet_size,
            wavelet_size,
            -wavelet_size,
            wavelet_size,
            -wavelet_size,
            wavelet_size,
        ]

        # update plane
        scaled_offset = self.clip.ClipType.Offset
        self.plane.Origin = [scaled_offset, -wavelet_size, -wavelet_size]
        self.plane.Point1 = [scaled_offset, -wavelet_size, wavelet_size]
        self.plane.Point2 = [scaled_offset, wavelet_size, -wavelet_size]

        self.ctrl.view_update()
        self._push_data_info()

    def ui_state_clip_update(self, clip_x_origin, **kwargs):
        if not self._ready:
            return

        minX, maxX, minY, maxY, minZ, maxZ = self.wavelet.WholeExtent
        scaled_offset = (maxX - minX) * clip_x_origin / 2.0

        # update plane
        self.plane.Origin = [scaled_offset, minY, minZ]
        self.plane.Point1 = [scaled_offset, minY, maxZ]
        self.plane.Point2 = [scaled_offset, maxY, minZ]

        # update clip
        self.clip.ClipType.Offset = scaled_offset

        self.ctrl.view_update()
        self._push_data_info()

    def ui_state_slice_update(self, slice_x_origin, **kwargs):
        if not self._ready:
            return

        minX, maxX, *_ = self.wavelet.WholeExtent
        scaled_offset = (maxX - minX) * slice_x_origin / 2.0
        self.slice.SliceType.Offset = scaled_offset

        self.ctrl.view_update()
        self._push_data_info()

    def ui_state_threshold_update(self, threshold_range, **kwargs):
        if not self._ready:
            return

        self.threshold.LowerThreshold = threshold_range[0]
        self.threshold.UpperThreshold = threshold_range[1]

        self.ctrl.view_update()
        self._push_data_info()

    def ui_state_target_fps_change(self, target_fps, **kwargs):
        if not self._ready:
            return
        # self._view_handler.target_fps = target_fps
        print(target_fps)


# -----------------------------------------------------------------------------
# Setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
app = App(server)
app.setup_demo()

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageWithDrawerLayout(server) as layout:
    with layout.icon:
        vuetify.VIcon("mdi-clock-time-four-outline")

    with layout.title as title:
        title.style = "padding-left: 0;"
        title.set_text("ParaView 5.11")

    with layout.toolbar as toolbar:
        toolbar.dense = True
        vuetify.VSpacer()
        vuetify.VProgressCircular(
            "S",
            color="red",
            size=35,
            width=5,
            rotate=("status_server", 0),
            value=("20",),
            classes="mx-2",
        )
        vuetify.VProgressCircular(
            "C",
            color="teal",
            size=35,
            width=5,
            indeterminate=True,
            classes="mx-2",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        vuetify.VCheckbox(
            small=True,
            v_model=("spinning", False),
            dense=True,
            classes="mx-2",
            hide_details=True,
            on_icon="mdi-axis-z-rotate-counterclockwise",
            off_icon="mdi-axis-z-rotate-counterclockwise",
        )
        with vuetify.VBtn(
            icon=True, small=True, click=app.ctrl.view_reset_camera, classes="mx-2"
        ):
            vuetify.VIcon("mdi-crop-free")

    with layout.drawer as drawer:
        drawer.width = 300
        trame.GitTree(
            sources=("git_tree_sources", []),
            actives=("git_tree_actives", []),
            visibility_change=(app.ui_event_pipeline_visibility_update, "[$event]"),
            actives_change=(app.ui_event_pipeline_update, "[$event]"),
        )
        with vuetify.VCard(
            classes="mb-2 mx-1", v_show="git_tree_actives.includes(wavelet_id)"
        ):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Wavelet")
                vuetify.VSpacer()
                html.Div("{{ wavelet_size }}")

            vuetify.VDivider()
            with vuetify.VCardText():
                vuetify.VSlider(
                    v_model=("wavelet_size", 10),
                    min=5,
                    max=100,
                    step=1,
                    hide_details=True,
                    dense=True,
                )
        with vuetify.VCard(
            classes="mb-2 mx-1", v_show="git_tree_actives.includes(contour_id)"
        ):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Contours")
                vuetify.VSpacer()
                html.Div("{{ contours_count }}")

            vuetify.VDivider()
            with vuetify.VCardText():
                vuetify.VSlider(
                    v_model=("contours_count", 10),
                    min=5,
                    max=100,
                    step=1,
                    hide_details=True,
                    dense=True,
                )
        with vuetify.VCard(
            classes="mb-2 mx-1", v_show="git_tree_actives.includes(clip_id)"
        ):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Clip")
                vuetify.VSpacer()
                html.Div("{{ clip_x_origin }}")

            vuetify.VDivider()
            with vuetify.VCardText():
                vuetify.VSlider(
                    v_model=("clip_x_origin", 0),
                    min=-1,
                    max=1,
                    step=0.1,
                    hide_details=True,
                    dense=True,
                )
        with vuetify.VCard(
            classes="mb-2 mx-1", v_show="git_tree_actives.includes(slice_id)"
        ):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Slice")
                vuetify.VSpacer()
                html.Div("{{ slice_x_origin }}")

            vuetify.VDivider()
            with vuetify.VCardText():
                vuetify.VSlider(
                    v_model=("slice_x_origin", 0),
                    min=-1,
                    max=1,
                    step=0.1,
                    hide_details=True,
                    dense=True,
                )
        with vuetify.VCard(
            classes="mb-2 mx-1", v_show="git_tree_actives.includes(threshold_id)"
        ):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Threshold")
                vuetify.VSpacer()
                html.Div("{{ threshold_range }}")
            vuetify.VDivider()
            with vuetify.VCardText():
                vuetify.VRangeSlider(
                    v_model=("threshold_range", (180, 240)),
                    min=WAVELET_SCALAR_RANGE[0],
                    max=WAVELET_SCALAR_RANGE[1],
                    step=0.5,
                    hide_details=True,
                    dense=True,
                )

        with vuetify.VCard(classes="mb-2 mx-1"):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Stats")
                vuetify.VSpacer()
                vuetify.VIcon("mdi-dots-triangle", x_small=True, classes="mr-1")
                html.Div(
                    r"{{ active_data_info?.points.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',') | 0 }}",
                    classes="text-caption",
                )
                vuetify.VSpacer()
                vuetify.VIcon("mdi-triangle-outline", x_small=True, classes="mr-1")
                html.Div(
                    r"{{ active_data_info?.cells.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',') || 0 }}",
                    classes="text-caption",
                )
            vuetify.VDivider()
            with vuetify.VCardText(style="height: 150px"):
                rca.StatisticsDisplay(
                    name="view",
                    fps_delta=1.5,
                    stat_window_size=10,
                    history_window_size=30,
                    reset_ms_threshold=100,
                    ws_topic="viewport.image.push.subscription",
                    packet_decorator=(
                        """
                        (v) => {
                            return {
                                name: 'view',
                                serverTime: Date.now(),
                                contentSize: v.memsize,
                            };
                        }
                    """,
                    ),
                )

        with vuetify.VCard(classes="my-2 mx-1"):
            with vuetify.VCardTitle(classes="py-0"):
                html.Div("Image Delivery")
            vuetify.VDivider()
            with vuetify.VCardText():
                html.Div("Target: {{ target_fps }} fps", classes="text-subtitle-2 mt-4")
                vuetify.VSlider(
                    v_model=("target_fps", 30),
                    min=10,
                    max=60,
                    step=5,
                    hide_details=True,
                    dense=True,
                )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            # rca.RemoteControlledArea(
            #     name="view", display=("active_display_mode", "image")
            # )
            v = paraview.VtkRemoteView(app.active_view, interactive_ratio=1)
            app.ctrl.view_reset_camera = v.reset_camera
            app.ctrl.view_update = v.update

# -----------------------------------------------------------------------------
# CLI
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import client, html, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------
VAR_NAMES = set(["a123", "a456", "a789"])

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

count = 800

state.update(
    {
        "items": [{"id": "a123"}, {"id": "a456"}, {"id": "a789"}],
        "a123": {
            "range": (10, 90),
            "range_min": 0,
            "range_max": 100,
            "opacity": 1,
        },
        "a456": {
            "range": (20, 80),
            "range_min": 0,
            "range_max": 100,
            "opacity": 1,
        },
        "a789": {
            "range": (30, 70),
            "range_min": 0,
            "range_max": 100,
            "opacity": 1,
        },
    }
)


def add_entry():
    # !!! Vue2 does not support new vars with getter but vue3 should handle it
    global count
    count += 1
    key = f"a{count}"
    state.items.append(dict(id=key))
    state[key] = (
        {
            "range": (0, 100),
            "range_min": 0,
            "range_max": 100,
            "opacity": 0.5,
        },
    )
    state.dirty("items")
    VAR_NAMES.add(key)
    state.change(key)(on_change)
    update_layout()


@state.change(*VAR_NAMES)
def on_change(**_):
    for name in state.modified_keys & VAR_NAMES:
        print(f"Data changed for {name}")
        print(state[name])
        print("-" * 30)

        if name == "a123":
            state.a456["range"] = list(state.a123["range"])
            state.dirty("a456")


def update_layout():
    with SinglePageWithDrawerLayout(server) as layout:
        with layout.toolbar:
            vuetify.VSpacer()
            vuetify.VBtn("Add", click=add_entry)
            vuetify.VBtn("Fast: {{fast}}", click="fast = !fast")
        with layout.drawer:
            with client.Getter(
                v_for="(item, i) in items",
                key="item.id",
                name=("`${item.id}`",),
                key_name="local_name",
                value_name="local_var",
                update_nested_name="update_local",
            ):
                with vuetify.VListItem():  # Must only be 1 child for a Getter
                    vuetify.VRangeSlider(
                        label=("`${local_name}`",),
                        value=("local_var.range",),
                        input="update_local('range', $event)",
                        min=("local_var.range_min",),
                        max=("local_var.range_max",),
                    )
                    vuetify.VSlider(
                        value=("local_var.opacity",),
                        input="update_local('opacity', $event)",
                        min=0,
                        max=1,
                        step=0.01,
                    )

        with layout.content:
            with vuetify.VRow():
                # With static HTML, the proper dependency get captured and state will print faster
                with vuetify.VCol(v_if=("fast", True)):
                    html.Div("Static dependencies", classes="text-h6")
                    for name in VAR_NAMES:
                        html.Pre("%s = {{ %s }}" % (name, name))

                # With method call in template, the refresh is debounced unless proper dep above
                with vuetify.VCol():
                    html.Div("Use method call", classes="text-h6")
                    with html.Div(
                        "State: {{ item.id }}",
                        v_for="item, idx in items",
                        key="item.id",
                    ):
                        html.Pre("{{ get(item.id) }}")


if __name__ == "__main__":
    update_layout()
    server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, vuetify
from trame.widgets import vtk as vtk_widgets

USING_WITH = True
server = get_server(client_type="vue2")
ctrl = server.controller


class Cone:
    def __init__(self, server, name):
        self.server = server
        self.name = name

        self.rw = vtkRenderWindow()
        renderer = vtkRenderer()
        self.rw.AddRenderer(renderer)
        cone_source = vtkConeSource()
        mapper = vtkPolyDataMapper()
        mapper.SetInputConnection(cone_source.GetOutputPort())
        actor = vtkActor()
        actor.SetMapper(mapper)
        renderer.AddActor(actor)
        renderer.ResetCamera()

        self.ui()

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    def ui(self, **kwargs):
        with vtk_widgets.VtkLocalView(self.rw, ref=f"view_{self.name}") as view:
            html.Div(
                self.name,
                style="position: absolute; left: 10px; top: 10px; z-index: 100; color: white;",
            )
            self.ctrl.view_update = view.update
            self.ctrl.view_reset_camera = view.reset_camera


# ---------------------------------------------------------
# Main UI
# ---------------------------------------------------------

with SinglePageLayout(server) as layout:
    with layout.toolbar:
        layout.title.set_text("Active Tab: {{ active_tab }}")
        layout.title.style = "min-width: 200px;"
        vuetify.VSpacer()
        with vuetify.VTabs(v_model=("active_tab", 0), right=True):
            for i in range(5):
                vuetify.VTab(f"Cone {i + 1}")

    with layout.content:
        with vuetify.VContainer(classes="fill-height", fluid=True):
            with vuetify.VTabsItems(
                value=("active_tab",), style="width: 100%; height: 100%;"
            ):
                for i in range(5):
                    with vuetify.VTabItem(
                        value=(i,), style="width: 100%; height: 100%;"
                    ):
                        # html.Div(f"Instance {i + 1}", style="background: red; width: 100%; height: 100%;")
                        Cone(server, f"cone_{i + 1}")


if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/ParaView/TimeAnimation/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/22329eec355476e5e1a5fb21dfb3b23ee02ae427

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

import asyncio
import json
import os

import paraview as pv
import paraview.web.venv  # Available in PV 5.10-RC2+
from paraview import simple

from trame.app import asynchronous, get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import paraview, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------

animation_scene = simple.GetAnimationScene()
time_keeper = animation_scene.TimeKeeper

metadata = None
time_values = []
representation = None


def load_data(**kwargs):
    global time_values, representation
    # CLI
    args, _ = server.cli.parse_known_args()
    full_path = os.path.abspath(args.data)
    base_path = os.path.dirname(full_path)
    files = []
    reader_props = {}

    with open(full_path, "r") as f:
        metadata = json.load(f)
        reader_props = metadata.get("reader_properties", {})
        fields = metadata.get("fields", [])
        for name in metadata.get("files", []):
            files.append(os.path.join(base_path, name))

    reader = simple.OpenDataFile(files)
    for key, value in reader_props.items():
        reader.GetProperty(key).SetData(value)

    reader.UpdatePipeline()
    representation = simple.Show(reader, view)
    time_values = list(time_keeper.TimestepValues)

    update_color_by(0, fields, "remote")
    state.time_value = time_values[0]
    state.times = len(time_values) - 1
    state.fields = fields

    simple.ResetCamera()
    view.CenterOfRotation = view.CameraFocalPoint
    update_view("local")


ctrl.on_server_ready.add(load_data)

# -----------------------------------------------------------------------------
# ParaView pipeline
# -----------------------------------------------------------------------------

# Rendering setup
view = simple.GetRenderView()
view.UseColorPaletteForBackground = 0
view.Background = [0.8, 0.8, 0.8]
view.OrientationAxesVisibility = 0
view = simple.Render()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("active_array")
def update_color_by(active_array, fields, viewMode="remote", **kwargs):
    if len(fields) == 0:
        return

    array = fields[active_array]
    simple.ColorBy(representation, (array.get("location"), array.get("text")))
    representation.RescaleTransferFunctionToDataRange(True, False)
    name = pv.make_name_valid(array.get("text"))
    lut = simple.GetColorTransferFunction(name)
    pwf = simple.GetOpacityTransferFunction(name)
    _min, _max = array.get("range")
    lut.RescaleTransferFunction(_min, _max)
    pwf.RescaleTransferFunction(_min, _max)
    update_view(viewMode)


@state.change("time")
def update_time(time, viewMode, **kwargs):
    if len(time_values) == 0:
        return

    if time >= len(time_values):
        time = 0
        state.time = time
    time_value = time_values[time]
    time_keeper.Time = time_value
    state.time_value = time_value
    update_view(viewMode)


@state.change("play")
@asynchronous.task
async def update_play(**kwargs):
    while state.play:
        with state:
            state.time += 1
            update_time(state.time, state.viewMode)

        await asyncio.sleep(0.1)


@state.change("viewMode")
def update_view(viewMode, **kwargs):
    ctrl.view_update_image()
    if viewMode == "local":
        ctrl.view_update_geometry()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

state.trame__title = "ParaView"

with SinglePageLayout(server) as layout:
    layout.title.set_text("Time")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSelect(
            v_model=("active_array", 0),
            items=("fields", []),
            style="max-width: 200px",
            hide_details=True,
            dense=True,
        )
        vuetify.VTextField(
            v_model=("time_value", 0),
            disabled=True,
            hide_details=True,
            dense=True,
            style="max-width: 200px",
            classes="mx-2",
        )
        vuetify.VSlider(
            v_model=("time", 0),
            min=0,
            max=("times", 1),
            hide_details=True,
            dense=True,
            style="max-width: 200px",
        )

        vuetify.VCheckbox(
            v_model=("play", False),
            off_icon="mdi-play",
            on_icon="mdi-stop",
            hide_details=True,
            dense=True,
            classes="mx-2",
        )

        with vuetify.VBtn(icon=True, click=ctrl.view_reset_camera):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VCheckbox(
            v_model=("viewMode", "remote"),
            true_value="remote",
            false_value="local",
            off_icon="mdi-rotate-3d",
            on_icon="mdi-video-image",
            hide_details=True,
            dense=True,
            classes="mx-2",
        )

        vuetify.VProgressLinear(
            indeterminate=True,
            absolute=True,
            bottom=True,
            active=("trame__busy",),
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            html_view = paraview.VtkRemoteLocalView(view, namespace="view")
            ctrl.view_update = html_view.update
            ctrl.view_update_geometry = html_view.update_geometry
            ctrl.view_update_image = html_view.update_image
            ctrl.view_reset_camera = html_view.reset_camera


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.cli.add_argument("--data", help="Path to state file", dest="data")
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-components
"""

from trame.app import get_server
from trame.ui.vuetify import VAppLayout
from trame.widgets import trame

selection = ["2"]
tree = [
    {"id": "1", "parent": "0", "visible": 0, "name": "Wavelet"},
    {"id": "2", "parent": "1", "visible": 0, "name": "Clip"},
    {"id": "3", "parent": "1", "visible": 1, "name": "Slice"},
    {"id": "4", "parent": "2", "visible": 1, "name": "Slice 2"},
]
server = get_server(client_type="vue2")

with VAppLayout(server):
    trame.GitTree(
        sources=("tree", tree),
        actives=("selection", selection),
    )

if __name__ == "__main__":
    server.start()



import pyvista as pv
from pyvista import examples
from pyvista.trame.ui import plotter_ui

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout

pv.OFF_SCREEN = True

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

mesh = examples.load_random_hills()

pl = pv.Plotter()
pl.add_mesh(mesh)

with SinglePageLayout(server) as layout:
    with layout.content:
        view = plotter_ui(pl)

server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from pathlib import Path

import paraview.web.venv
from paraview import simple

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import paraview, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Data file
# -----------------------------------------------------------------------------

data_directory = Path(__file__).parent.parent.parent.with_name("data")
head_vti = data_directory / "head.vti"

# -----------------------------------------------------------------------------
# ParaView pipeline
# -----------------------------------------------------------------------------
simple.LoadDistributedPlugin("AcceleratedAlgorithms", remote=False, ns=globals())
reader = simple.XMLImageDataReader(FileName=[str(head_vti)])
# contour = simple.Contour(Input=reader) # Default filter    => no plugin but slow
contour = simple.FlyingEdges3D(Input=reader)  # Faster processing => make it interactive

# Extract data range => Update store/state
array = reader.GetPointDataInformation().GetArray(0)
data_name = array.GetName()
data_range = array.GetRange()
contour_value = 0.5 * (data_range[0] + data_range[1])
state.data_range = data_range
state.contour_value = contour_value

contour.ContourBy = ["POINTS", data_name]
contour.Isosurfaces = [contour_value]
contour.ComputeNormals = 1
contour.ComputeScalars = 0

# Rendering setup
view = simple.GetRenderView()
view.OrientationAxesVisibility = 0
representation = simple.Show(contour, view)
view = simple.Render()
simple.ResetCamera()
view.CenterOfRotation = view.CameraFocalPoint

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("contour_value")
def update_contour(contour_value, **kwargs):
    contour.Isosurfaces = [contour_value]
    html_view.update_image()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

modes = (
    ("auto", "mdi-autorenew"),
    ("local", "mdi-rotate-3d"),
    ("remote", "mdi-image"),
)

state.trame__title = "ParaView contour - Remote/Local rendering"
ctrl.on_server_ready.add(ctrl.view_update)

with SinglePageLayout(server) as layout:
    layout.title.set_text("Contour Application - Remote rendering")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar:
        vuetify.VSpacer()

        with vuetify.VBtnToggle(
            v_model=("override", "auto"),
            dense=True,
            mandatory=True,
        ):
            for entry in modes:
                with vuetify.VBtn(value=entry[0]):
                    vuetify.VIcon(entry[1])

        vuetify.VSpacer()
        vuetify.VSlider(
            v_model="contour_value",
            min=("data_range[0]",),
            max=("data_range[1]",),
            hide_details=True,
            dense=True,
            style="max-width: 300px",
            start="trigger('demoAnimateStart')",
            end="trigger('demoAnimateStop')",
        )
        vuetify.VSwitch(
            v_model="$vuetify.theme.dark",
            hide_details=True,
        )

        with vuetify.VBtn(icon=True, click=ctrl.view_reset_camera):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VProgressLinear(
            indeterminate=True,
            absolute=True,
            bottom=True,
            active=("trame__busy",),
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            html_view = paraview.VtkRemoteLocalView(
                view,
                namespace="demo",
                # second arg is to force the view to start in "local" mode
                mode=("override === 'auto' ? demoMode : override", "local"),
            )
            ctrl.view_update = html_view.update
            ctrl.view_reset_camera = html_view.reset_camera

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from pathlib import Path

import paraview.web.venv
from paraview import simple

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import client, paraview, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller
state.trame__title = "State Viewer"

# Preload paraview modules onto server
paraview.initialize(server)

# -----------------------------------------------------------------------------
# ParaView code
# -----------------------------------------------------------------------------


@ctrl.add("on_server_ready")
def load_data(**kwargs):
    # CLI
    args, _ = server.cli.parse_known_args()

    full_path = str(Path(args.data).resolve().absolute())
    working_directory = str(Path(args.data).parent.resolve().absolute())

    # ParaView
    simple.LoadState(
        full_path,
        data_directory=working_directory,
        restrict_to_data_directory=True,
    )
    view = simple.GetActiveView()
    view.MakeRenderWindowInteractor(True)
    simple.Render(view)

    # HTML
    with SinglePageLayout(server) as layout:
        layout.icon.click = ctrl.view_reset_camera
        layout.title.set_text("ParaView State Viewer")

        with layout.content:
            with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
                html_view = paraview.VtkRemoteView(view)
                ctrl.view_reset_camera = html_view.reset_camera
                ctrl.view_update = html_view.update


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("ParaView State Viewer")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            client.Loading("Loading state")


if __name__ == "__main__":
    server.cli.add_argument("--data", help="Path to state file", dest="data")
    server.start()



from pathlib import Path

from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import client, vuetify3


def load_my_js(server):
    # must run before server start
    js_file = Path(__file__).with_name("my_utils.js").resolve()
    server.enable_module(
        dict(
            # Path to serve under /my_code
            serve={"my_code": str(js_file.parent)},
            # JS file(s) to load
            scripts=[f"my_code/{js_file.name}"],
        )
    )


class CustomAddOnJS:
    def __init__(self, server=None, table_size=10):
        self.server = get_server(server, client_type="vue3")
        self.ui = self._build_ui()

        # Make sure my js code get loaded on the client side
        load_my_js(self.server)

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    def trigger_alert(self):
        self.ctrl.js_eval(f"server({self.state.name})")

    def _build_ui(self):
        with SinglePageLayout(self.server, full_height=True) as layout:
            self.ctrl.js_eval = client.JSEval(
                exec="utils.my_code.actions.hello($event)"
            ).exec
            with layout.toolbar:
                vuetify3.VTextField(
                    v_model=("name", "world"),
                    density="compact",
                    hide_details=True,
                )
                vuetify3.VBtn("Local JS", click="utils.my_code.actions.hello(name)")
                vuetify3.VBtn("JS from Py", click=self.trigger_alert)
            with layout.content:
                vuetify3.VTextField(
                    v_model=("text_1", "1.2"),
                    # Use our extended trame.utils in template definition
                    rules=("[utils.my_code.rules.number]",),
                )
                vuetify3.VTextField(
                    v_model=("text_2", "2"),
                    # Use our extended trame.utils in template definition
                    rules=("[utils.my_code.rules.int]",),
                )


def main():
    app = CustomAddOnJS()
    app.server.start()


if __name__ == "__main__":
    main()



from pathlib import Path

from trame.app import get_server
from trame.assets.local import LocalFileManager
from trame.ui.html import DivLayout
from trame.widgets import html

KEYS = []
DIRECTORY = Path(__file__).parent.parent.parent / "docs/vitepress/assets/images/apps"
ASSETS = LocalFileManager(DIRECTORY)

for image in DIRECTORY.iterdir():
    key = image.stem.replace("-", "_")
    ASSETS.url(key, image.name)
    KEYS.append(key)


server = get_server()
with DivLayout(server):
    for key in KEYS:
        html.Img(src=ASSETS[key], style="width: 200px;")


if __name__ == "__main__":
    server.start()



# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.router import RouterViewLayout
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import html, router, trame, vtk, vuetify

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor = vtkActor()
actor.SetMapper(mapper)

renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()


# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone_source.SetResolution(resolution)
    ctrl.view_update()


layout = SinglePageWithDrawerLayout(server)

# There are two ways to register a route
with RouterViewLayout(server, "/"):
    trame.LifeCycleMonitor(name="Home", events="['created', 'destroyed']")
    with vuetify.VCard():
        vuetify.VCardTitle("This is home {{ resolution }}")

with RouterViewLayout(server, "/foo"):
    trame.LifeCycleMonitor(name="Foo", events="['created', 'destroyed']")
    with vuetify.VCard():
        vuetify.VCardTitle("This is foo")
        with vuetify.VCardText():
            vuetify.VBtn("Take me back {{ resolution }}", click="$router.back()")
            with html.Div(classes="mt-6", style="height: 400px; min-height: 400px;"):
                view = vtk.VtkLocalView(renderWindow)
                view.update()
                ctrl.view_update.add(view.update)

# or use the contextmanager 'with_route'
with RouterViewLayout(server, "/bar/:id"):
    trame.LifeCycleMonitor(name="Bar", events="['created', 'destroyed']")
    with vuetify.VCard():
        vuetify.VCardTitle(
            "This is bar with ID '{{ $route.params.id }}' with resolution {{ resolution }}"
        )
        with vuetify.VCardText(classes="red lighten-5 pa-0"):
            with html.Div(style="height: 400px; min-height: 400px;"):
                view = vtk.VtkRemoteView(renderWindow)
                view.update()
                ctrl.view_update.add(view.update)
            with html.Div(style="height: 400px; min-height: 400px;"):
                view = vtk.VtkLocalView(renderWindow)
                view.update()
                ctrl.view_update.add(view.update)

# add <router-view />
with layout:
    layout.title.set_text("Multi-Page demo")
    trame.LifeCycleMonitor(name="Root", events="['created', 'destroyed']")

    with layout.toolbar as tb:
        vuetify.VSpacer()
        tb.add_child("{{ resolution }}")
        vuetify.VSlider(
            v_model=("resolution", 6),
            min=3,
            max=60,
            step=1,
            dense=True,
            hide_details=True,
            style="max-width: 300px;",
        )

    with layout.content:
        with vuetify.VContainer():
            router.RouterView()

    # add router buttons to the drawer
    with layout.drawer:
        with vuetify.VList(shaped=True, v_model=("selectedRoute", 0)):
            vuetify.VSubheader("Routes")

            with vuetify.VListItem(to="/"):
                with vuetify.VListItemIcon():
                    vuetify.VIcon("mdi-home")
                with vuetify.VListItemContent():
                    vuetify.VListItemTitle("Home")

            with vuetify.VListItem(to="/foo"):
                with vuetify.VListItemIcon():
                    vuetify.VIcon("mdi-food")
                with vuetify.VListItemContent():
                    vuetify.VListItemTitle("Foo")

            with vuetify.VListGroup(value=("true",), sub_group=True):
                with vuetify.Template(v_slot_activator=True):
                    vuetify.VListItemTitle("Bars")
                with vuetify.VListItemContent():
                    for i in range(5):
                        with vuetify.VListItem(to=f"/bar/{i + 1}"):
                            with vuetify.VListItemIcon():
                                vuetify.VIcon("mdi-peanut-outline")
                            with vuetify.VListItemContent():
                                vuetify.VListItemTitle("Bar {{ resolution }}")
                                vuetify.VListItemSubtitle(f"ID {i + 1}")

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    ctrl.view_update()
    server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa

# -----------------------------------------------------------------------------
# VTK
# -----------------------------------------------------------------------------
from vtkmodules.vtkFiltersSources import vtkConeSource, vtkSphereSource
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html, vtk

DEFAULT_RESOLUTION = 6

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()


def show(source):
    mapper = vtkPolyDataMapper()
    actor = vtkActor()
    mapper.SetInputConnection(source.GetOutputPort())
    actor.SetMapper(mapper)
    renderer.AddActor(actor)
    return source


cone = show(vtkConeSource())
sphere = show(vtkSphereSource())

renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

with DivLayout(server) as layout:
    container = layout.root
    container.style = "width: 100vw; height: 100vh;"
    view = vtk.VtkRemoteView(renderWindow, interactive_ratio=1, interactive_quality=0)
    ctrl.view_update = view.update
    html.Input(
        type="range",
        min=8,
        max=1000,
        step=1,
        v_model=("resolution", 8),
        style="position: absolute; top: 20px; left: 20px; z-index: 1; width: 25%; min-width: 300px;",
    )


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone.SetResolution(int(resolution))
    sphere.SetPhiResolution(int(resolution))
    sphere.SetThetaResolution(int(resolution))
    ctrl.view_update()


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import VAppLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")


@server.state.change("value")
def update_number(value, **kwargs):
    print(f"{type(value)}({value})")


with VAppLayout(server):
    vuetify.VTextField(v_model_number=("value", 1), type="number")

server.start()



import vtk

from trame.app import TrameApp
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vuetify3, vtk as vtk_widgets


class PushCamera(TrameApp):
    def __init__(self, server=None):
        super().__init__(server)
        self._setup_vtk()
        self._build_ui()

    def _setup_vtk(self):
        render_window = vtk.vtkRenderWindow()
        render_window.SetOffScreenRendering(True)

        renderer = vtk.vtkRenderer()
        render_window.AddRenderer(renderer)

        cone = vtk.vtkConeSource()
        mapper = vtk.vtkPolyDataMapper()
        actor = vtk.vtkActor()

        mapper.SetInputConnection(cone.GetOutputPort())
        actor.SetMapper(mapper)
        renderer.AddActor(actor)
        renderer.ResetCamera()

        self.render_window = render_window
        self.camera = renderer.GetActiveCamera()

    def _build_ui(self):
        with SinglePageLayout(self.server) as self.ui:
            self.ui.title.set_text("Client-side Camera Control")

            with self.ui.content:
                with vuetify3.VContainer(
                    fluid=True,
                    classes="pa-0 fill-height",
                ):
                    vtk_widgets.VtkLocalView(self.render_window, ctx_name="view")

            with self.ui.toolbar:
                vuetify3.VSpacer()
                vuetify3.VBtn("Set Top View", click=self.set_top_view)

    def set_top_view(self):
        self.camera.SetPosition(0, 5, 0)
        self.camera.SetFocalPoint(0, 0, 0)
        self.camera.SetViewUp(0, 0, -1)

        self.ctx.view.push_camera()
        # self.ctx.view.update()


if __name__ == "__main__":
    app = PushCamera()
    app.server.start()



import plotly.express as px

from trame.app import get_server
from trame.ui.router import RouterViewLayout
from trame.ui.vuetify3 import SinglePageWithDrawerLayout
from trame.widgets import plotly, router
from trame.widgets import vuetify3 as vuetify

# -----------------------------------------------------------------------------
# Charts
# -----------------------------------------------------------------------------


def scatter():
    df = px.data.iris()

    template = "plotly"
    if state.theme_mode == "dark":
        template = "plotly_dark"

    fig = px.scatter(
        df,
        x="sepal_width",
        y="sepal_length",
        color="species",
        template=template,
        title="A Plotly Express Figure",
    )

    return fig


# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server()
server.client_type = "vue3"
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("theme_mode")
def update_cube_axes_visibility(**kwargs):
    ctrl.graph_update(scatter())


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

# Home route
with RouterViewLayout(server, "/"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is home")
        with vuetify.VContainer(fluid=True, style="height: 600px;"):
            with vuetify.VRow(dense=True):
                vuetify.VSpacer()
                figure = plotly.Figure(
                    display_logo=False,
                    display_mode_bar="true",
                    # selected=(on_event, "['selected', utils.safe($event)]"),
                    # hover=(on_event, "['hover', utils.safe($event)]"),
                    # selecting=(on_event, "['selecting', $event]"),
                    # unhover=(on_event, "['unhover', $event]"),
                )
                ctrl.graph_update = figure.update
                vuetify.VSpacer()

# Foo route
with RouterViewLayout(server, "/foo"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is foo")
        with vuetify.VCardText():
            vuetify.VBtn("Take me back", click="$router.back()")

# Bar/id
with RouterViewLayout(server, "/bar/:id"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is bar with ID '{{ $route.params.id }}'")


# Main page content
with SinglePageWithDrawerLayout(server, theme=("theme_mode", "light")) as layout:
    layout.title.set_text("Multi-Page demo")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VCheckbox(
            v_model=("theme_mode", "light"),
            true_icon="mdi-lightbulb-off-outline",
            false_icon="mdi-lightbulb-outline",
            true_value="dark",
            false_value="light",
            classes="mx-1",
            hide_details=True,
            dense=True,
        )
        # vuetify.VCheckbox(
        #     v_model="$vuetify.theme.dark",
        #     color="white",
        #     change=("graph_theme", "[$event]"),
        #     on_icon="mdi-lightbulb-off-outline",
        #     off_icon="mdi-lightbulb-outline",
        #     classes="mx-1",
        #     hide_details=True,
        #     dense=True,
        # )

    with layout.content:
        with vuetify.VContainer():
            router.RouterView()

    # add router buttons to the drawer
    with layout.drawer:
        with vuetify.VList(shaped=True, v_model=("selectedRoute", 0)):
            # vuetify.VSubheader("Routes")

            with vuetify.VListItem(to="/"):
                vuetify.VIcon("mdi-home")
                vuetify.VListItemTitle("Home")

            with vuetify.VListItem(to="/foo"):
                vuetify.VIcon("mdi-food")
                vuetify.VListItemTitle("Foo")

            with vuetify.VListGroup(value=("true",), sub_group=True):
                with vuetify.Template(v_slot_activator=True):
                    vuetify.VListItemTitle("Bars")
                # with vuetify.VListItemContent():
                #     with vuetify.VListItem(v_for="id in [1]", to=("'/bar/' + id",)):
                #         # with vuetify.VListItemIcon():
                #         #     vuetify.VIcon("mdi-peanut-outline")
                #         with vuetify.VListItemContent():
                #             vuetify.VListItemTitle("Bar")
                #             vuetify.VListItemSubtitle("ID '{{id}}'")

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------


def main():
    server.start()


if __name__ == "__main__":
    main()



import os
from pathlib import Path

import vtk

# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOLegacy import vtkUnstructuredGridReader
from vtkmodules.vtkRenderingCore import (
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk2
from trame.widgets import vuetify

CURRENT_DIRECTORY = os.path.abspath(os.path.dirname(__file__))

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)
renderWindow.OffScreenRenderingOn()

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()


# Read the data

# reader = vtkStructuredPointsReader()
reader = vtkUnstructuredGridReader()

reader.SetFileName(str(Path(__file__).with_name("lv_fiber.vtk").absolute()))
reader.Update()
# 2. 创建箭头源
arrow_source = vtk.vtkArrowSource()
arrow_source.Update()
# 3. 创建箭头映射器
arrow_mapper = vtk.vtkGlyph3DMapper()
arrow_mapper.SetInputConnection(reader.GetOutputPort())
arrow_mapper.SetSourceConnection(arrow_source.GetOutputPort())
arrow_mapper.SetScaleFactor(1.0)  # 调整箭头大小
arrow_mapper.ScalingOn()
arrow_mapper.Update()
# 4. 创建箭头演员
arrow_actor = vtk.vtkActor()
arrow_actor.SetMapper(arrow_mapper)
# 7. 添加箭头演员到渲染器
renderer.AddActor(arrow_actor)
# 8. 设置渲染器背景颜色和相机位置
renderer.SetBackground(1.0, 1.0, 1.0)
# 9. 设置箭头及轴的尺寸
arrow_source.SetTipLength(0.1)  # 箭头尖部的长度
arrow_source.SetTipRadius(0.05)  # 箭头尖部的半径
arrow_source.SetShaftRadius(0.02)  # 箭头杆的半径

renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
ctrl = server.controller

with SinglePageLayout(server) as layout:
    layout.title.set_text("Hello1 trame")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk2.VtkRemoteView(renderWindow, interactive_ratio=1)
            # view = vtk2.VtkLocalView(renderWindow)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import client, html


class DeepReactiveArray:
    def __init__(self, server=None):
        self.server = get_server(server)

        self.state.data = {}

        with DivLayout(self.server) as self.ui:
            html.Button("Add", click=self.add)
            html.Button("Remove", click=self.remove)
            with client.DeepReactive("data"):
                with html.Ul():
                    with html.Li(
                        key="k",
                        v_for="v, k in data",
                    ):
                        html.Span("{{ v.name }}")
                        html.Input(
                            type="range",
                            min=0,
                            max=100,
                            step=1,
                            v_model="data[k].slider",
                        )

            html.Pre("{{ JSON.stringify(data, null, 2) }}")

    @property
    def state(self):
        return self.server.state

    def add(self):
        name = f"Item {len(self.state.data)}"
        self.state.data[name] = {"name": name, "slider": 50}
        self.state.dirty("data")

    def remove(self):
        last_key = f"Item {len(self.state.data) - 1}"
        self.state.data.pop(last_key)
        self.state.dirty("data")
        # self.state.data = { k:v for k,v in self.state.data.items() if k != last_key }


def main():
    app = DeepReactiveArray()
    app.server.start()


if __name__ == "__main__":
    main()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/ContourGeometry/RemoteViewLocalRendering.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/3852cba56cd63f6efa684d5e5fb00881a6111e81

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from pathlib import Path

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersCore import vtkContourFilter

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOXML import vtkXMLImageDataReader
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK contour - Remote/Local rendering"

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

data_directory = Path(__file__).parent.parent.with_name("data")
head_vti = data_directory / "head.vti"

reader = vtkXMLImageDataReader()
reader.SetFileName(head_vti)
reader.Update()

contour = vtkContourFilter()
contour.SetInputConnection(reader.GetOutputPort())
contour.SetComputeNormals(1)
contour.SetComputeScalars(0)

# Extract data range => Update store/state
data_range = reader.GetOutput().GetPointData().GetScalars().GetRange()
contour_value = 0.5 * (data_range[0] + data_range[1])
state.data_range = data_range

# Configure contour with valid values
contour.SetNumberOfContours(1)
contour.SetValue(0, contour_value)

# Rendering setup
renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(contour.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("contour_value", "interactive")
def update_contour(contour_value, interactive, force=False, **kwargs):
    if interactive or force:
        contour.SetValue(0, contour_value)
        ctrl.view_update()


def commit_changes():
    update_contour(
        contour_value=state.contour_value,
        interactive=state.interactive,
        force=True,
    )


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.title.set_text("Contour Application - Remote rendering")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSwitch(
            v_model=("interactive", False),
            hide_details=True,
            label="Update while dragging",
        )
        vuetify.VSlider(
            v_model=("contour_value", contour_value),
            change=commit_changes,
            min=("data_range[0]",),
            max=("data_range[1]",),
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VSwitch(
            v_model="$vuetify.theme.dark",
            hide_details=True,
        )
        with vuetify.VBtn(
            icon=True,
            click=ctrl.view_reset_camera,
        ):
            vuetify.VIcon("mdi-crop-free")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk_widgets.VtkLocalView(renderWindow)
            # view = vtk_widgets.VtkRemoteView(renderWindow)
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera
            ctrl.on_server_ready.add(view.update)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import matplotlib.pyplot as plt
import io
import base64

from trame.app import TrameApp
from trame.widgets import html, vuetify3 as v3
from trame.ui.vuetify3 import SinglePageLayout


def text_to_base64_png(
    text: str, width: int = 600, height: int = 200, dpi: int = 100
) -> str:
    # Convert pixels to inches for figsize
    figsize = (width / dpi, height / dpi)

    # Create the figure
    fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
    ax.axis("off")  # Hide axes

    # Add text at the center
    ax.text(0.5, 0.5, text, fontsize=14, ha="center", va="center")

    # Save to buffer
    buf = io.BytesIO()
    plt.savefig(buf, format="png", bbox_inches="tight", dpi=dpi)
    plt.close(fig)
    buf.seek(0)

    # Convert to base64
    img_base64 = base64.b64encode(buf.read()).decode("ascii")
    return f"data:image/png;base64,{img_base64}"


# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------


class TabImages(TrameApp):
    def __init__(self, name=None):
        super().__init__(server=name)

        self.state.left_logo_img = text_to_base64_png(
            "Left Logo Image", width=512, height=488
        )
        self.state.right_logo_img = text_to_base64_png(
            "Right Logo Image", width=490, height=300
        )
        self.state.main_canvas_left_img = text_to_base64_png(
            "Left Main Canvas Image", width=700, height=488
        )
        self.state.main_canvas_right_img = text_to_base64_png(
            "Right Main Canvas Image", width=700, height=488
        )
        self.build_ui()

    def build_ui(self):
        with SinglePageLayout(self.server) as layout:
            layout.title.set_text("Dummy GUI")
            layout.icon.hide()

            # Toolbar with left and right logos
            with layout.toolbar:
                with v3.VRow(classes="align-center"):
                    with v3.VCol(cols=6):
                        html.Img(src=("left_logo_img",), style="height: 80px;")
                    with v3.VCol(cols=6, classes="d-flex justify-end"):
                        html.Img(src=("right_logo_img",), style="height: 80px;")

            # Tabs
            with layout.content:
                with v3.VTabs(v_model=("active_tab", "main"), grow=True):
                    v3.VTab("Main", value="main")
                    v3.VTab("Logic", value="logic")
                    v3.VTab("Overview", value="overview")

                with v3.VTabsWindow(v_model="active_tab"):
                    # Main tab
                    with v3.VTabsWindowItem(value="main"):
                        with v3.VContainer(fluid=True):
                            with v3.VRow():
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_left_img",),
                                        style="width: 100%; height: auto;",
                                    )
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_right_img",),
                                        style="width: 100%; height: auto;",
                                    )

                    # Logic tab
                    with v3.VTabsWindowItem(value="logic"):
                        with v3.VContainer(fluid=True):
                            with v3.VRow():
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_left_img",),
                                        style="width: 100%; height: auto;",
                                    )
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_right_img",),
                                        style="width: 100%; height: auto;",
                                    )

                    # Overview tab
                    with v3.VTabsWindowItem(value="overview"):
                        with v3.VContainer(fluid=True):
                            with v3.VRow():
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_left_img",),
                                        style="width: 100%; height: auto;",
                                    )
                                with v3.VCol(cols=12, md=6):
                                    html.Img(
                                        src=("main_canvas_right_img",),
                                        style="width: 100%; height: auto;",
                                    )


if __name__ == "__main__":
    app = TabImages()
    app.server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/Applications/MultiFilter/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/7b3cda7c45b8a60132892f762e16ed28cb709481

Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-components
"""

import os

# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa
from trame_vtk.modules.vtk.serializers import configure_serializer
from vtkmodules.vtkCommonDataModel import vtkDataObject
from vtkmodules.vtkFiltersCore import vtkContourFilter

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader
from vtkmodules.vtkRenderingAnnotation import vtkCubeAxesActor
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import trame, vuetify
from trame.widgets import vtk as vtk_widgets

# Configure scene encoder
configure_serializer(encode_lut=True, skip_light=True)

CURRENT_DIRECTORY = os.path.abspath(os.path.dirname(__file__))

# -----------------------------------------------------------------------------
# Constants
# -----------------------------------------------------------------------------


class Representation:
    Points = 0
    Wireframe = 1
    Surface = 2
    SurfaceWithEdges = 3


class LookupTable:
    Rainbow = 0
    Inverted_Rainbow = 1
    Greyscale = 2
    Inverted_Greyscale = 3


# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

# Read Data
reader = vtkXMLUnstructuredGridReader()
reader.SetFileName(os.path.join(CURRENT_DIRECTORY, "../../../data/disk_out_ref.vtu"))
reader.Update()

# Extract Array/Field information
dataset_arrays = []
fields = [
    (reader.GetOutput().GetPointData(), vtkDataObject.FIELD_ASSOCIATION_POINTS),
    (reader.GetOutput().GetCellData(), vtkDataObject.FIELD_ASSOCIATION_CELLS),
]
for field in fields:
    field_arrays, association = field
    for i in range(field_arrays.GetNumberOfArrays()):
        array = field_arrays.GetArray(i)
        array_range = array.GetRange()
        dataset_arrays.append(
            {
                "text": array.GetName(),
                "value": i,
                "range": list(array_range),
                "type": association,
            }
        )
default_array = dataset_arrays[0]
default_min, default_max = default_array.get("range")

# Mesh
mesh_mapper = vtkDataSetMapper()
mesh_mapper.SetInputConnection(reader.GetOutputPort())
mesh_actor = vtkActor()
mesh_actor.SetMapper(mesh_mapper)
renderer.AddActor(mesh_actor)

# Mesh: Setup default representation to surface
mesh_actor.GetProperty().SetRepresentationToSurface()
mesh_actor.GetProperty().SetPointSize(1)
mesh_actor.GetProperty().EdgeVisibilityOff()

# Mesh: Apply rainbow color map
mesh_lut = mesh_mapper.GetLookupTable()
mesh_lut.SetHueRange(0.666, 0.0)
mesh_lut.SetSaturationRange(1.0, 1.0)
mesh_lut.SetValueRange(1.0, 1.0)
mesh_lut.Build()

# Mesh: Color by default array
mesh_mapper.SelectColorArray(default_array.get("text"))
mesh_mapper.GetLookupTable().SetRange(default_min, default_max)
if default_array.get("type") == vtkDataObject.FIELD_ASSOCIATION_POINTS:
    mesh_mapper.SetScalarModeToUsePointFieldData()
else:
    mesh_mapper.SetScalarModeToUseCellFieldData()
mesh_mapper.SetScalarVisibility(True)
mesh_mapper.SetUseLookupTableScalarRange(True)

# Contour
contour = vtkContourFilter()
contour.SetInputConnection(reader.GetOutputPort())
contour_mapper = vtkDataSetMapper()
contour_mapper.SetInputConnection(contour.GetOutputPort())
contour_actor = vtkActor()
contour_actor.SetMapper(contour_mapper)
renderer.AddActor(contour_actor)

# Contour: ContourBy default array
contour_value = 0.5 * (default_max + default_min)
contour.SetInputArrayToProcess(
    0, 0, 0, default_array.get("type"), default_array.get("text")
)
contour.SetValue(0, contour_value)

# Contour: Setup default representation to surface
contour_actor.GetProperty().SetRepresentationToSurface()
contour_actor.GetProperty().SetPointSize(1)
contour_actor.GetProperty().EdgeVisibilityOff()

# Contour: Apply rainbow color map
contour_lut = contour_mapper.GetLookupTable()
contour_lut.SetHueRange(0.666, 0.0)
contour_lut.SetSaturationRange(1.0, 1.0)
contour_lut.SetValueRange(1.0, 1.0)
contour_lut.Build()

# Contour: Color by default array
contour_mapper.SelectColorArray(default_array.get("text"))
contour_mapper.GetLookupTable().SetRange(default_min, default_max)
if default_array.get("type") == vtkDataObject.FIELD_ASSOCIATION_POINTS:
    contour_mapper.SetScalarModeToUsePointFieldData()
else:
    contour_mapper.SetScalarModeToUseCellFieldData()
contour_mapper.SetScalarVisibility(True)
contour_mapper.SetUseLookupTableScalarRange(True)

# Cube Axes
cube_axes = vtkCubeAxesActor()
renderer.AddActor(cube_axes)

# Cube Axes: Boundaries, camera, and styling
cube_axes.SetBounds(mesh_actor.GetBounds())
cube_axes.SetCamera(renderer.GetActiveCamera())
cube_axes.SetXLabelFormat("%6.1f")
cube_axes.SetYLabelFormat("%6.1f")
cube_axes.SetZLabelFormat("%6.1f")
cube_axes.SetFlyModeToOuterEdges()

renderer.ResetCamera()

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.setdefault("active_ui", None)

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("cube_axes_visibility")
def update_cube_axes_visibility(cube_axes_visibility, **kwargs):
    cube_axes.SetVisibility(cube_axes_visibility)
    ctrl.view_update()


# Selection Change
def actives_change(ids):
    _id = ids[0]
    if _id == "1":  # Mesh
        state.active_ui = "mesh"
    elif _id == "2":  # Contour
        state.active_ui = "contour"
    else:
        state.active_ui = "nothing"


# Visibility Change
def visibility_change(event):
    _id = event["id"]
    _visibility = event["visible"]

    if _id == "1":  # Mesh
        mesh_actor.SetVisibility(_visibility)
    elif _id == "2":  # Contour
        contour_actor.SetVisibility(_visibility)
    ctrl.view_update()


# Representation Callbacks
def update_representation(actor, mode):
    property = actor.GetProperty()
    if mode == Representation.Points:
        property.SetRepresentationToPoints()
        property.SetPointSize(5)
        property.EdgeVisibilityOff()
    elif mode == Representation.Wireframe:
        property.SetRepresentationToWireframe()
        property.SetPointSize(1)
        property.EdgeVisibilityOff()
    elif mode == Representation.Surface:
        property.SetRepresentationToSurface()
        property.SetPointSize(1)
        property.EdgeVisibilityOff()
    elif mode == Representation.SurfaceWithEdges:
        property.SetRepresentationToSurface()
        property.SetPointSize(1)
        property.EdgeVisibilityOn()


@state.change("mesh_representation")
def update_mesh_representation(mesh_representation, **kwargs):
    update_representation(mesh_actor, mesh_representation)
    ctrl.view_update()


@state.change("contour_representation")
def update_contour_representation(contour_representation, **kwargs):
    update_representation(contour_actor, contour_representation)
    ctrl.view_update()


# Color By Callbacks
def color_by_array(actor, array):
    _min, _max = array.get("range")
    mapper = actor.GetMapper()
    mapper.SelectColorArray(array.get("text"))
    mapper.GetLookupTable().SetRange(_min, _max)
    if array.get("type") == vtkDataObject.FIELD_ASSOCIATION_POINTS:
        mesh_mapper.SetScalarModeToUsePointFieldData()
    else:
        mesh_mapper.SetScalarModeToUseCellFieldData()
    mapper.SetScalarVisibility(True)
    mapper.SetUseLookupTableScalarRange(True)


@state.change("mesh_color_array_idx")
def update_mesh_color_by_name(mesh_color_array_idx, **kwargs):
    array = dataset_arrays[mesh_color_array_idx]
    color_by_array(mesh_actor, array)
    ctrl.view_update()


@state.change("contour_color_array_idx")
def update_contour_color_by_name(contour_color_array_idx, **kwargs):
    array = dataset_arrays[contour_color_array_idx]
    color_by_array(contour_actor, array)
    ctrl.view_update()


# Color Map Callbacks
def use_preset(actor, preset):
    lut = actor.GetMapper().GetLookupTable()
    if preset == LookupTable.Rainbow:
        lut.SetHueRange(0.666, 0.0)
        lut.SetSaturationRange(1.0, 1.0)
        lut.SetValueRange(1.0, 1.0)
    elif preset == LookupTable.Inverted_Rainbow:
        lut.SetHueRange(0.0, 0.666)
        lut.SetSaturationRange(1.0, 1.0)
        lut.SetValueRange(1.0, 1.0)
    elif preset == LookupTable.Greyscale:
        lut.SetHueRange(0.0, 0.0)
        lut.SetSaturationRange(0.0, 0.0)
        lut.SetValueRange(0.0, 1.0)
    elif preset == LookupTable.Inverted_Greyscale:
        lut.SetHueRange(0.0, 0.666)
        lut.SetSaturationRange(0.0, 0.0)
        lut.SetValueRange(1.0, 0.0)
    lut.Build()


@state.change("mesh_color_preset")
def update_mesh_color_preset(mesh_color_preset, **kwargs):
    use_preset(mesh_actor, mesh_color_preset)
    ctrl.view_update()


@state.change("contour_color_preset")
def update_contour_color_preset(contour_color_preset, **kwargs):
    use_preset(contour_actor, contour_color_preset)
    ctrl.view_update()


# Opacity Callbacks
@state.change("mesh_opacity")
def update_mesh_opacity(mesh_opacity, **kwargs):
    mesh_actor.GetProperty().SetOpacity(mesh_opacity)
    ctrl.view_update()


@state.change("contour_opacity")
def update_contour_opacity(contour_opacity, **kwargs):
    contour_actor.GetProperty().SetOpacity(contour_opacity)
    ctrl.view_update()


# Contour Callbacks
@state.change("contour_by_array_idx")
def update_contour_by(contour_by_array_idx, **kwargs):
    array = dataset_arrays[contour_by_array_idx]
    contour_min, contour_max = array.get("range")
    contour_step = 0.01 * (contour_max - contour_min)
    contour_value = 0.5 * (contour_max + contour_min)
    contour.SetInputArrayToProcess(0, 0, 0, array.get("type"), array.get("text"))
    contour.SetValue(0, contour_value)

    # Update UI
    state.contour_min = contour_min
    state.contour_max = contour_max
    state.contour_value = contour_value
    state.contour_step = contour_step

    # Update View
    ctrl.view_update()


@state.change("contour_value")
def update_contour_value(contour_value, **kwargs):
    contour.SetValue(0, float(contour_value))
    ctrl.view_update()


# -----------------------------------------------------------------------------
# GUI elements
# -----------------------------------------------------------------------------


def standard_buttons():
    vuetify.VCheckbox(
        v_model=("cube_axes_visibility", True),
        on_icon="mdi-cube-outline",
        off_icon="mdi-cube-off-outline",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model="$vuetify.theme.dark",
        on_icon="mdi-lightbulb-off-outline",
        off_icon="mdi-lightbulb-outline",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("viewMode", "local"),
        on_icon="mdi-lan-disconnect",
        off_icon="mdi-lan-connect",
        true_value="local",
        false_value="remote",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    with vuetify.VBtn(icon=True, click="$refs.view.resetCamera()"):
        vuetify.VIcon("mdi-crop-free")


def pipeline_widget():
    trame.GitTree(
        sources=(
            "pipeline",
            [
                {"id": "1", "parent": "0", "visible": 1, "name": "Mesh"},
                {"id": "2", "parent": "1", "visible": 1, "name": "Contour"},
            ],
        ),
        actives_change=(actives_change, "[$event]"),
        visibility_change=(visibility_change, "[$event]"),
    )


def ui_card(title, ui_name):
    with vuetify.VCard(v_show=f"active_ui == '{ui_name}'"):
        vuetify.VCardTitle(
            title,
            classes="grey lighten-1 py-1 grey--text text--darken-3",
            style="user-select: none; cursor: pointer",
            hide_details=True,
            dense=True,
        )
        content = vuetify.VCardText(classes="py-2")
    return content


def mesh_card():
    with ui_card(title="Mesh", ui_name="mesh"):
        vuetify.VSelect(
            # Representation
            v_model=("mesh_representation", Representation.Surface),
            items=(
                "representations",
                [
                    {"text": "Points", "value": 0},
                    {"text": "Wireframe", "value": 1},
                    {"text": "Surface", "value": 2},
                    {"text": "SurfaceWithEdges", "value": 3},
                ],
            ),
            label="Representation",
            hide_details=True,
            dense=True,
            outlined=True,
            classes="pt-1",
        )
        with vuetify.VRow(classes="pt-2", dense=True):
            with vuetify.VCol(cols="6"):
                vuetify.VSelect(
                    # Color By
                    label="Color by",
                    v_model=("mesh_color_array_idx", 0),
                    items=("array_list", dataset_arrays),
                    hide_details=True,
                    dense=True,
                    outlined=True,
                    classes="pt-1",
                )
            with vuetify.VCol(cols="6"):
                vuetify.VSelect(
                    # Color Map
                    label="Colormap",
                    v_model=("mesh_color_preset", LookupTable.Rainbow),
                    items=(
                        "colormaps",
                        [
                            {"text": "Rainbow", "value": 0},
                            {"text": "Inv Rainbow", "value": 1},
                            {"text": "Greyscale", "value": 2},
                            {"text": "Inv Greyscale", "value": 3},
                        ],
                    ),
                    hide_details=True,
                    dense=True,
                    outlined=True,
                    classes="pt-1",
                )
        vuetify.VSlider(
            # Opacity
            v_model=("mesh_opacity", 1.0),
            min=0,
            max=1,
            step=0.1,
            label="Opacity",
            classes="mt-1",
            hide_details=True,
            dense=True,
        )


def contour_card():
    with ui_card(title="Contour", ui_name="contour"):
        vuetify.VSelect(
            # Contour By
            label="Contour by",
            v_model=("contour_by_array_idx", 0),
            items=("array_list", dataset_arrays),
            hide_details=True,
            dense=True,
            outlined=True,
            classes="pt-1",
        )
        vuetify.VSlider(
            # Contour Value
            v_model=("contour_value", contour_value),
            min=("contour_min", default_min),
            max=("contour_max", default_max),
            step=("contour_step", 0.01 * (default_max - default_min)),
            label="Value",
            classes="my-1",
            hide_details=True,
            dense=True,
        )
        vuetify.VSelect(
            # Representation
            v_model=("contour_representation", Representation.Surface),
            items=(
                "representations",
                [
                    {"text": "Points", "value": 0},
                    {"text": "Wireframe", "value": 1},
                    {"text": "Surface", "value": 2},
                    {"text": "SurfaceWithEdges", "value": 3},
                ],
            ),
            label="Representation",
            hide_details=True,
            dense=True,
            outlined=True,
            classes="pt-1",
        )
        with vuetify.VRow(classes="pt-2", dense=True):
            with vuetify.VCol(cols="6"):
                vuetify.VSelect(
                    # Color By
                    label="Color by",
                    v_model=("contour_color_array_idx", 0),
                    items=("array_list", dataset_arrays),
                    hide_details=True,
                    dense=True,
                    outlined=True,
                    classes="pt-1",
                )
            with vuetify.VCol(cols="6"):
                vuetify.VSelect(
                    # Color Map
                    label="Colormap",
                    v_model=("contour_color_preset", LookupTable.Rainbow),
                    items=(
                        "colormaps",
                        [
                            {"text": "Rainbow", "value": 0},
                            {"text": "Inv Rainbow", "value": 1},
                            {"text": "Greyscale", "value": 2},
                            {"text": "Inv Greyscale", "value": 3},
                        ],
                    ),
                    hide_details=True,
                    dense=True,
                    outlined=True,
                    classes="pt-1",
                )
        vuetify.VSlider(
            # Opacity
            v_model=("contour_opacity", 1.0),
            min=0,
            max=1,
            step=0.1,
            label="Opacity",
            classes="mt-1",
            hide_details=True,
            dense=True,
        )


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageWithDrawerLayout(server) as layout:
    layout.title.set_text("Viewer")

    with layout.toolbar:
        # toolbar components
        vuetify.VSpacer()
        vuetify.VDivider(vertical=True, classes="mx-2")
        standard_buttons()

    with layout.drawer as drawer:
        # drawer components
        drawer.width = 325
        pipeline_widget()
        vuetify.VDivider(classes="mb-2")
        mesh_card()
        contour_card()

    with layout.content:
        # content components
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            # view = vtk_widgets.VtkRemoteView(renderWindow, interactive_ratio=1)
            # view = vtk_widgets.VtkLocalView(renderWindow)
            view = vtk_widgets.VtkRemoteLocalView(
                renderWindow, namespace="view", mode="local", interactive_ratio=1
            )
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera
            ctrl.on_server_ready.add(view.update)

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/PlainPython/GeoMaps/UberPickupsNYC/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/6f0882ab9060b0f1953915c2a7e93dbe996479e0

An example of showing geographic data

Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-deckgl trame-vega
"""

import os

import altair as alt
import numpy as np
import pandas as pd
import pydeck as pdk

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import deckgl, html, vega, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# LOADING DATA
# -----------------------------------------------------------------------------
DATE_TIME = "date/time"
DATA_URL = (
    "http://s3-us-west-2.amazonaws.com/streamlit-demo-data/uber-raw-data-sep14.csv.gz"
)


def lowercase(x):
    return str(x).lower()


def load_data(nrows):
    data = pd.read_csv(DATA_URL, nrows=nrows)
    data.rename(lowercase, axis="columns", inplace=True)
    data[DATE_TIME] = pd.to_datetime(data[DATE_TIME])
    return data


data = load_data(100000)

# CREATING FUNCTION FOR MAPS
map_list = [
    {
        "id": "nyc",
        "title": "All New York City",
        "lat": np.average(data["lat"]),
        "lon": np.average(data["lon"]),
        "zoom": 11,
    },
    {
        "id": "lga",
        "title": "La Guardia Airport",
        "lat": 40.7900,
        "lon": -73.8700,
        "zoom": 12,
    },
    {
        "id": "jfk",
        "title": "JFK Airport",
        "lat": 40.6650,
        "lon": -73.7821,
        "zoom": 11,
    },
    {
        "id": "nwk",
        "title": "Newark Airport",
        "lat": 40.7090,
        "lon": -74.1805,
        "zoom": 11,
    },
]


def updateMap(data, id, lat, lon, zoom, **kwarg):
    deck = pdk.Deck(
        map_provider="mapbox",
        map_style="mapbox://styles/mapbox/light-v9",
        initial_view_state={
            "latitude": lat,
            "longitude": lon,
            "zoom": zoom,
            "pitch": 50,
        },
        layers=[
            pdk.Layer(
                "HexagonLayer",
                data=data,
                get_position=["lon", "lat"],
                radius=100,
                elevation_scale=4,
                elevation_range=[0, 1000],
                pickable=True,
                extruded=True,
            ),
        ],
    )
    ctrl[f"{id}_update"](deck)
    state[f"{id}Title"] = kwarg["title"]


@state.change("pickupHour")
def updateData(pickupHour, **kwargs):
    state.chartTitle = f"All New York City from {pickupHour}:00 and {pickupHour + 1}:00"

    # FILTERING DATA BY HOUR SELECTED
    filtered_data = data[data[DATE_TIME].dt.hour == pickupHour]

    for item in map_list:
        updateMap(filtered_data, **item)

    # FILTERING DATA FOR THE HISTOGRAM
    filtered = filtered_data[
        (data[DATE_TIME].dt.hour >= pickupHour)
        & (data[DATE_TIME].dt.hour < (pickupHour + 1))
    ]

    hist = np.histogram(filtered[DATE_TIME].dt.minute, bins=60, range=(0, 60))[0]

    chart_data = pd.DataFrame({"minute": range(60), "pickups": hist})

    # LAYING OUT THE HISTOGRAM SECTION
    ctrl.hour_breakdown_update(
        alt.Chart(chart_data)
        .mark_area(
            interpolate="step-after",
        )
        .properties(width="container", height=150)
        .encode(
            x=alt.X("minute:Q", scale=alt.Scale(nice=False)),
            y=alt.Y("pickups:Q"),
            tooltip=["minute", "pickups"],
        )
        .configure_mark(opacity=0.5, color="red")
    )


# -----------------------------------------------------------------------------
# GUI Layout
# -----------------------------------------------------------------------------

dynamicTitle = "{{nycTitle}} from {{pickupHour}}:00 and {{pickupHour + 1}}:00"

mapProps = {
    "classes": "elevation-5",
    "mapboxApiKey": os.environ["MAPBOX_API_KEY"],
    "style": "height: 50vh;",
}

with SinglePageLayout(server) as layout:
    layout.title.set_text("NYC Uber Ridesharing Data")

    with layout.content:
        with vuetify.VContainer(fluid="true") as container:
            html.Div(
                """Examining how Uber pickups vary over time in New York City's
                    and at its major regional airports.
                    By sliding the slider on the left you can view different slices
                    of time and explore different transportation trends.""",
                classes="text-body-1",
            )
            vuetify.VSlider(
                v_model=("pickupHour", 0),
                classes="mt-4",
                label="Select hour of pickup",
                min=0,
                max=23,
                thumb_label=True,
            )

            with vuetify.VRow():
                with vuetify.VCol(cols=4):
                    html.Div(dynamicTitle, classes="text-h5")
                    ctrl.nyc_update = deckgl.Deck(**mapProps).update
                with vuetify.VCol(cols=8) as col:
                    with vuetify.VRow():
                        for title, name in [
                            ("{{jfkTitle}}", "jfk"),
                            ("{{lgaTitle}}", "lga"),
                            ("{{nwkTitle}}", "nwk"),
                        ]:
                            with vuetify.VCol(cols=4):
                                html.Div(title, classes="text-h5")
                                ctrl[f"{name}_update"] = deckgl.Deck(**mapProps).update

            with html.Div(classes="text-center mt-6"):
                ctrl.hour_breakdown_update = vega.Figure(style="width: 100%").update


# -----------------------------------------------------------------------------
# Start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import asyncio

# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkCommonColor import vtkNamedColors
from vtkmodules.vtkFiltersSources import vtkConeSource, vtkSphereSource

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import (
    vtkInteractorStyleSwitch,  # noqa
    vtkInteractorStyleTrackballCamera,
)
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import VAppLayout
from trame.widgets import html, vtk, vuetify

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------


colors = vtkNamedColors()


def cone():
    cone = vtkConeSource()
    cone.SetResolution(60)

    # Create a mapper and actor
    mapper = vtkPolyDataMapper()
    mapper.SetInputConnection(cone.GetOutputPort())

    actor = vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetDiffuseColor(colors.GetColor3d("bisque"))

    # Visualize
    renderer = vtkRenderer()
    renderWindow = vtkRenderWindow()
    renderWindow.AddRenderer(renderer)
    renderWindowInteractor = vtkRenderWindowInteractor()
    interactor_style = vtkInteractorStyleTrackballCamera()
    renderWindowInteractor.SetInteractorStyle(interactor_style)
    renderWindowInteractor.SetRenderWindow(renderWindow)

    renderer.AddActor(actor)
    renderer.SetBackground(colors.GetColor3d("bisque"))
    renderer.ResetCamera()

    renderWindow.SetSize(640, 480)
    renderWindow.SetWindowName("Cone")

    return renderWindow


def sphere():
    sphere = vtkSphereSource()

    # Create a mapper and actor
    mapper = vtkPolyDataMapper()
    mapper.SetInputConnection(sphere.GetOutputPort())

    actor = vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetDiffuseColor(colors.GetColor3d("bisque"))

    # Visualize
    renderer = vtkRenderer()
    renderWindow = vtkRenderWindow()
    renderWindow.AddRenderer(renderer)
    renderWindowInteractor = vtkRenderWindowInteractor()
    interactor_style = vtkInteractorStyleTrackballCamera()
    renderWindowInteractor.SetInteractorStyle(interactor_style)
    renderWindowInteractor.SetRenderWindow(renderWindow)

    renderer.AddActor(actor)
    renderer.SetBackground(colors.GetColor3d("Salmon"))
    renderer.ResetCamera()

    renderWindow.SetSize(640, 480)
    renderWindow.SetWindowName("Sphere")

    return renderWindow


cone_window = cone()
sphere_window = sphere()

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.visible_view = 0


# -----------------------------------------------------------------------------
# Background thread
# -----------------------------------------------------------------------------


async def refresh_function(**kwargs):
    while True:
        with state:
            state.visible_view += 1
            ctrl.view_update()

        await asyncio.sleep(5)


ctrl.on_server_ready.add_task(refresh_function)

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------


with VAppLayout(server) as layout:
    with layout.root:
        with vuetify.VContainer(
            fluid=True, classes="pa-0 fill-height", style="position: relative;"
        ):
            with html.Div(
                style="z-index: 1; width; 100%; height: 100%;",
                v_if="!(visible_view % 2)",
            ):
                view = vtk.VtkLocalView(
                    cone_window,
                    context_name="cone",
                    namespace="cone",
                    ref="cone",
                    style="position: absolute; width: 100%; height: 100%;",
                )
                ctrl.view_update.add(view.update)
            with html.Div(
                style="z-index: 1; width; 100%; height: 100%;", v_if="visible_view % 2"
            ):
                view = vtk.VtkLocalView(
                    sphere_window,
                    context_name="sphere",
                    namespace="sphere",
                    ref="sphere",
                    style="position: absolute; width: 100%; height: 100%;",
                )
                ctrl.view_update.add(view.update)

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()
